@@ -18,10 +18,13 @@
 
 import os
 import time
+import pstats
 import pickle
 import getpass
+import tempfile
 import operator
 import itertools
+import collections
 from datetime import datetime
 from decorator import decorator
 import psutil
@@ -48,6 +51,35 @@ task_info_dt = numpy.dtype(
 
 I64 = numpy.int64
 
+PStatData = collections.namedtuple(
+    'PStatData', 'ncalls tottime percall cumtime percall2 path')
+
+
+def get_pstats(pstatfile, n):
+    """
+    Return profiling information as a list [(ncalls, cumtime, path), ...]
+
+    :param pstatfile: path to a .pstat file
+    :param n: the maximum number of stats to retrieve
+    """
+    with tempfile.TemporaryFile(mode='w+') as stream:
+        ps = pstats.Stats(pstatfile, stream=stream)
+        ps.sort_stats('cumtime')
+        ps.print_stats(n)
+        stream.seek(0)
+        lines = list(stream)
+    for i, line in enumerate(lines):
+        if line.startswith('   ncalls'):
+            break
+    data = []
+    for line in lines[i + 2:]:
+        columns = line.split()
+        if len(columns) == 6:
+            columns[-1] = os.path.basename(columns[-1])
+            data.append(PStatData(*columns))
+    rows = [(rec.ncalls, rec.cumtime, rec.path) for rec in data]
+    return rows
+
 
 def init_performance(hdf5file, swmr=False):
     """
@@ -149,11 +181,12 @@ class Monitor(object):
     calc_id = None
 
     def __init__(self, operation='', measuremem=False, inner_loop=False,
-                 h5=None):
+                 h5=None, version=None):
         self.operation = operation
         self.measuremem = measuremem
         self.inner_loop = inner_loop
         self.h5 = h5
+        self.version = version
         self.mem = 0
         self.duration = 0
         self._start_time = self._stop_time = time.time()
