@@ -31,10 +31,46 @@ from openquake.calculators import base, views
 
 U8 = numpy.uint8
 F32 = numpy.float32
+F64 = numpy.float64
 U16 = numpy.uint16
 U32 = numpy.uint32
 
 
+def save_curve_stats(dstore):
+    """
+    Save agg_curves-stats
+    """
+    oq = dstore['oqparam']
+    units = dstore['cost_calculator'].get_units(oq.loss_types)
+    try:
+        K = len(dstore['agg_keys'])
+    except KeyError:
+        K = 0
+    stats = oq.hazard_stats()
+    S = len(stats)
+    weights = dstore['weights'][:]
+    aggcurves_df = dstore.read_df('aggcurves')
+    periods = aggcurves_df.return_period.unique()
+    P = len(periods)
+    for lt in oq.ext_loss_types:
+        loss_id = scientific.LTI[lt]
+        out = numpy.zeros((K + 1, S, P))
+        aggdf = aggcurves_df[aggcurves_df.loss_id == loss_id]
+        for agg_id, df in aggdf.groupby("agg_id"):
+            for s, stat in enumerate(stats.values()):
+                for p in range(P):
+                    dfp = df[df.return_period == periods[p]]
+                    ws = weights[dfp.rlz_id.to_numpy()]
+                    ws /= ws.sum()
+                    out[agg_id, s, p] = stat(dfp.loss.to_numpy(), ws)
+        stat = 'agg_curves-stats/' + lt
+        dstore.create_dset(stat, F64, (K + 1, S, P))
+        dstore.set_shape_descr(stat, agg_id=K+1, stat=list(stats),
+                               return_period=periods)
+        dstore.set_attrs(stat, units=units)
+        dstore[stat][:] = out
+
+
 def reagg_idxs(num_tags, tagnames):
     """
     :param num_tags: dictionary tagname -> number of tags with that tagname
@@ -112,23 +148,25 @@ def get_loss_builder(dstore, return_periods=None, loss_dt=None,
         eff_time, oq.risk_investigation_time or oq.investigation_time)
 
 
-def get_src_loss_table(dstore, L):
+def get_src_loss_table(dstore, loss_id):
     """
     :returns:
-        (source_ids, array of losses of shape (Ns, L))
+        (source_ids, array of losses of shape Ns)
     """
     K = dstore['risk_by_event'].attrs.get('K', 0)
-    alt = dstore.read_df('risk_by_event', 'agg_id', dict(agg_id=K))
+    alt = dstore.read_df('risk_by_event', 'agg_id',
+                         dict(agg_id=K, loss_id=loss_id))
+    if len(alt) == 0:  # no losses for this loss type
+        return [], ()
     eids = alt.event_id.to_numpy()
     evs = dstore['events'][:][eids]
     rlz_ids = evs['rlz_id']
     rup_ids = evs['rup_id']
     source_id = python3compat.decode(dstore['ruptures']['source_id'][rup_ids])
     w = dstore['weights'][:]
-    acc = general.AccumDict(accum=numpy.zeros(L, F32))
-    for source_id, rlz_id, loss_id, loss in zip(
-            source_id, rlz_ids, alt.loss_id.to_numpy(), alt.loss.to_numpy()):
-        acc[source_id][loss_id] += loss * w[rlz_id]
+    acc = general.AccumDict(accum=0)
+    for src, rlz_id, loss in zip(source_id, rlz_ids, alt.loss.to_numpy()):
+        acc[src] += loss * w[rlz_id]
     return zip(*sorted(acc.items()))
 
 
@@ -179,11 +217,13 @@ def store_agg(dstore, rbe_df, num_events):
     Build the aggrisk and aggcurves tables from the risk_by_event table
     """
     oq = dstore['oqparam']
-    if oq.investigation_time:
-        T = oq.investigation_time * oq.ses_per_logic_tree_path
     size = dstore.getsize('risk_by_event')
     logging.info('Building aggrisk from %s of risk_by_event',
                  general.humansize(size))
+    if oq.investigation_time:  # event based
+        tr = oq.time_ratio  # (risk_invtime / haz_invtime) * num_ses
+        if oq.collect_rlzs:  # reduce the time ratio by the number of rlzs
+            tr /= len(dstore['weights'])
     rlz_id = dstore['events']['rlz_id']
     if len(num_events) > 1:
         rbe_df['rlz_id'] = rlz_id[rbe_df.event_id.to_numpy()]
@@ -208,7 +248,7 @@ def store_agg(dstore, rbe_df, num_events):
             aggrisk['dmg_0'].append(aggnumber[agg_id] - ndamaged / ne)
         for col in columns:
             agg = df[col].sum()
-            aggrisk[col].append(agg/T if oq.investigation_time else agg/ne)
+            aggrisk[col].append(agg * tr if oq.investigation_time else agg/ne)
     fix_dtypes(aggrisk)
     aggrisk = pandas.DataFrame(aggrisk)
     dstore.create_df(
@@ -275,11 +315,12 @@ class PostRiskCalculator(base.RiskCalculator):
         if 'source_info' in self.datastore and 'risk' in oq.calculation_mode:
             logging.info('Building the src_loss_table')
             with self.monitor('src_loss_table', measuremem=True):
-                source_ids, losses = get_src_loss_table(self.datastore, self.L)
-                self.datastore['src_loss_table'] = losses
-                self.datastore.set_shape_descr('src_loss_table',
-                                               source=source_ids,
-                                               loss_type=oq.loss_types)
+                for loss_type in oq.loss_types:
+                    source_ids, losses = get_src_loss_table(
+                        self.datastore, scientific.LTI[loss_type])
+                    self.datastore['src_loss_table/' + loss_type] = losses
+                    self.datastore.set_shape_descr(
+                        'src_loss_table/' + loss_type, source=source_ids)
         K = len(self.datastore['agg_keys']) if oq.aggregate_by else 0
         rbe_df = self.datastore.read_df('risk_by_event')
         if self.reaggreate:
@@ -294,13 +335,14 @@ class PostRiskCalculator(base.RiskCalculator):
 
     def post_execute(self, dummy):
         """
-        Sanity checks
+        Sanity checks and save agg_curves-stats
         """
         oq = self.oqparam
         # logging.info('Total portfolio loss\n' +
         #              views.view('portfolio_loss', self.datastore))
         if oq.investigation_time and 'risk' in oq.calculation_mode:
-            for li, ln in enumerate(self.oqparam.loss_types):
+            for ln in self.oqparam.loss_types:
+                li = scientific.LTI[ln]
                 dloss = views.view('delta_loss:%d' % li, self.datastore)
                 if dloss['delta'].mean() > .1:  # more than 10% variation
                     logging.warning(
@@ -313,18 +355,33 @@ class PostRiskCalculator(base.RiskCalculator):
                    'addition-is-non-associative.html')
             K = len(self.datastore['agg_keys']) if oq.aggregate_by else 0
             aggrisk = self.aggrisk[self.aggrisk.agg_id == K]
-            avg_losses = self.datastore['avg_losses-rlzs'][:].sum(axis=0)
+            avg_losses = {
+                lt: self.datastore['avg_losses-rlzs/' + lt][:].sum(axis=0)
+                for lt in oq.loss_types}
             # shape (R, L)
             for _, row in aggrisk.iterrows():
                 ri, li = int(row.rlz_id), int(row.loss_id)
+                lt = scientific.LOSSTYPE[li]
+                if lt not in avg_losses:
+                    continue
                 # check on the sum of the average losses
-                avg = avg_losses[ri, li]
+                avg = avg_losses[lt][ri]
                 agg = row.loss
+                if not numpy.allclose(avg, agg, rtol=.1):
+                    # a serious discrepancy is an error
+                    raise ValueError("agg != sum(avg) [%s]: %s %s" %
+                                     (lt, agg, avg))
                 if not numpy.allclose(avg, agg, rtol=.001):
+                    # a small discrepancy is expected
                     logging.warning(
                         'Due to rounding errors inherent in floating-point '
                         'arithmetic, agg_losses != sum(avg_losses) [%s]: '
-                        '%s != %s\nsee %s', oq.loss_types[li], agg, avg, url)
+                        '%s != %s\nsee %s', lt, agg, avg, url)
+
+        # save agg_curves-stats
+        if (self.R > 1 and 'aggcurves' in self.datastore and
+                'risk' in oq.calculation_mode):
+            save_curve_stats(self.datastore)
 
 
 def post_aggregate(calc_id: int, aggregate_by):
