@@ -33,9 +33,10 @@ from openquake.baselib.general import (
 from openquake.baselib.hdf5 import FLOAT, INT, get_shape_descr
 from openquake.baselib.performance import performance_view
 from openquake.baselib.python3compat import encode, decode
+from openquake.hazardlib.contexts import KNOWN_DISTANCES
 from openquake.hazardlib.gsim.base import ContextMaker, Collapser
 from openquake.commonlib import util, logictree
-from openquake.risklib.scientific import losses_by_period, return_periods
+from openquake.risklib.scientific import losses_by_period, return_periods, LTI
 from openquake.baselib.writers import build_header, scientificformat
 from openquake.calculators.getters import get_rupture_getters
 from openquake.calculators.extract import extract
@@ -175,14 +176,15 @@ def view_worst_sources(token, dstore):
     else:
         step = 1
     data = dstore.read_df('source_data', 'src_id')
-    del data['nsites']
+    del data['impact']
     ser = data.groupby('taskno').ctimes.sum().sort_values().tail(1)
     [[taskno, maxtime]] = ser.to_dict().items()
     data = data[data.taskno == taskno]
-    print('Sources in the slowest task (%d seconds, weight=%d)'
-          % (maxtime, data['weight'].sum()))
+    print('Sources in the slowest task (%d seconds, weight=%d, taskno=%d)'
+          % (maxtime, data['weight'].sum(), taskno))
     data['slow_rate'] = data.ctimes / data.weight
-    df = data.sort_values('slow_rate', ascending=False)
+    del data['taskno']
+    df = data.sort_values('ctimes', ascending=False)
     return df[slice(None, None, step)]
 
 
@@ -192,8 +194,7 @@ def view_slow_sources(token, dstore, maxrows=20):
     Returns the slowest sources
     """
     info = dstore['source_info']['source_id', 'code',
-                                 'calc_time', 'num_sites', 'eff_ruptures']
-    info = info[info['eff_ruptures'] > 0]
+                                 'calc_time', 'num_sites', 'num_ruptures']
     info.sort(order='calc_time')
     data = numpy.zeros(len(info), dt(info.dtype.names))
     for name in info.dtype.names:
@@ -254,6 +255,7 @@ def view_full_lt(token, dstore):
 def view_eff_ruptures(token, dstore):
     info = dstore.read_df('source_info', 'source_id')
     df = info.groupby('code').sum()
+    df['slow_factor'] = df.calc_time / df.weight
     del df['grp_id'], df['trti']
     return df
 
@@ -362,7 +364,7 @@ def view_totlosses(token, dstore):
     sanity check for the correctness of the implementation.
     """
     oq = dstore['oqparam']
-    tot_losses = dstore['avg_losses-rlzs'][()].sum(axis=0)
+    tot_losses = dstore['avg_losses-rlzs/structural'][()].sum(axis=0)
     return text_table(tot_losses.view(oq.loss_dt(F32)), fmt='%.6E')
 
 
@@ -375,10 +377,10 @@ def alt_to_many_columns(alt, loss_types):
         dic[ln] = []
     for (eid, kid), df in alt.groupby(['event_id', 'agg_id']):
         dic['event_id'].append(eid)
-        arr = numpy.zeros(len(loss_types))
-        arr[df.loss_id.to_numpy()] = df.loss.to_numpy()
-        for li, ln in enumerate(loss_types):
-            dic[ln].append(arr[li])
+        for ln in loss_types:
+            arr = df[df.loss_id == LTI[ln]].loss.to_numpy()
+            loss = 0 if len(arr) == 0 else arr[0]  # arr has size 0 or 1
+            dic[ln].append(loss)
     return pandas.DataFrame(dic)
 
 
@@ -428,8 +430,8 @@ def view_portfolio_loss(token, dstore):
     E = len(rlzs)
     ws = weights[rlzs]
     avgs = []
-    for li, ln in enumerate(oq.loss_types):
-        df = alt_df[alt_df.loss_id == li]
+    for ln in oq.loss_types:
+        df = alt_df[alt_df.loss_id == LTI[ln]]
         eids = df.pop('event_id').to_numpy()
         avgs.append(ws[eids] @ df.loss.to_numpy() / ws.sum() * E / R)
     return text_table([['avg'] + avgs], ['loss'] + oq.loss_types)
@@ -1234,13 +1236,13 @@ def view_agg_id(token, dstore):
     """
     Show the available aggregations
     """
-    dfa = dstore.read_df('agg_keys')
-    keys = [col for col in dfa.columns if not col.endswith('_')]
-    df = dfa[keys]
-    totdf = pandas.DataFrame({key: ['*total*'] for key in keys})
-    concat = pandas.concat([df, totdf], ignore_index=True)
-    concat.index.name = 'agg_id'
-    return concat
+    [aggby] = dstore['oqparam'].aggregate_by
+    keys = [key.decode('utf8').split(',') for key in dstore['agg_keys'][:]]
+    keys = numpy.array(keys)  # shape (N, A)
+    dic = {aggkey: keys[:, a] for a, aggkey in enumerate(aggby)}
+    df = pandas.DataFrame(dic)
+    df.index.name = 'agg_id'
+    return df
 
 
 @view.add('mean_perils')
@@ -1305,25 +1307,20 @@ def view_collapsible(token, dstore):
     """
     Show how much the ruptures are collapsed for each site
     """
-    def recarray(mag, rrups, vs30s, dtype=dt('mag rrup vs30')):
-        out = [(mag, rrups[sid], vs30) for sid, vs30 in enumerate(vs30s)]
-        return numpy.array(out, dtype).view(numpy.recarray)
-
-    sitecol = dstore['sitecol']
-    rup_arr = dstore['rup/id'][:]
-    mag_arr = dstore['rup/mag'][:]
-    rrup_arr = dstore['rup/rrup_'][:]
-    sids_arr = dstore['rup/sids_'][:]
-    c1 = Collapser(1)
-    dic = dict(rup_id=[], site_id=[], mdvbin=[])
-    for id, mag, rrup, sids in zip(rup_arr, mag_arr, rrup_arr, sids_arr):
-        mdvbin = c1.calc_mdvbin(recarray(mag, rrup, sitecol.vs30[sids]))
-        for sid, mdv in zip(sids, mdvbin):
-            dic['rup_id'].append(id)
-            dic['site_id'].append(sid)
-            dic['mdvbin'].append(mdv)
+    if ':' in token:
+        collapse_level = int(token.split(':')[1])
+    else:
+        collapse_level = 0
+    dist_types = [dt for dt in dstore['rup'] if dt in KNOWN_DISTANCES]
+    vs30 = dstore['sitecol'].vs30
+    ctx_df = dstore.read_df('rup')
+    ctx_df['vs30'] = vs30[ctx_df.sids]
+    has_vs30 = len(numpy.unique(vs30)) > 1
+    c = Collapser(collapse_level, dist_types, has_vs30)
+    ctx_df['mdvbin'] = c.calc_mdvbin(ctx_df)
+    print('cfactor = %d/%d' % (len(ctx_df), len(ctx_df['mdvbin'].unique())))
     out = []
-    for sid, df in pandas.DataFrame(dic).groupby('site_id'):
+    for sid, df in ctx_df.groupby('sids'):
         n, u = len(df), len(df.mdvbin.unique())
         out.append((sid, u, n, n / u))
     return numpy.array(out, dt('site_id eff_rups num_rups cfactor'))
