@@ -31,7 +31,7 @@ import numpy
 from openquake.baselib import __version__, hdf5, python3compat, config
 from openquake.baselib.general import DictArray, AccumDict
 from openquake.hazardlib.imt import from_string
-from openquake.hazardlib.shakemap.maps import get_array
+from openquake.hazardlib import shakemap
 from openquake.hazardlib import correlation, cross_correlation, stats, calc
 from openquake.hazardlib import valid, InvalidFile, site
 from openquake.sep.classes import SecondaryPeril
@@ -226,6 +226,11 @@ distance_bin_width:
 ebrisk_maxsize:
   INTERNAL
 
+epsilon_star:
+  A boolean controlling the typology of disaggregation output to be provided.
+  When True disaggregation is perfomed in terms of epsilon* rather then
+  epsilon (see Bazzurro and Cornell, 1999)
+
 floating_x_step:
   Float, used in rupture generation for kite faults. indicates the fraction
   of fault length used to float ruptures along strike by the given float
@@ -669,6 +674,9 @@ source_id:
    Example: *source_id = src001 src002*.
    Default: empty list
 
+source_nodes:
+  INTERNAL
+
 spatial_correlation:
   Used in the ShakeMap calculator. The choics are "yes", "no" and "full".
   Example: *spatial_correlation = full*.
@@ -707,10 +715,18 @@ time_per_task:
   Example: *time_per_task=600*
   Default: 2000
 
+total_losses:
+  Used in event based risk calculations to compute total losses and
+  and total curves by summing across different loss types. Possible values
+  are "structural+nonstructural", "structural+contents",
+  "nonstructural+contents", "structural+nonstructural+contents".
+  Example: *total_losses = structural+nonstructural*
+  Default: None
+
 truncation_level:
   Truncation level used in the GMPEs.
   Example: *truncation_level = 0* to compute median GMFs.
-  Default: None
+  Default: 99
 
 uniform_hazard_spectra:
   Flag used to generated uniform hazard specta for the given poes
@@ -755,7 +771,8 @@ ALL_CALCULATORS = ['classical_risk',
                    'preclassical',
                    'conditional_spectrum',
                    'event_based_damage',
-                   'scenario_damage']
+                   'scenario_damage',
+                   'reinsurance_risk']
 
 
 def check_same_levels(imtls):
@@ -786,7 +803,8 @@ class OqParam(valid.ParamSet):
     _input_files = ()  # set in get_oqparam
     KNOWN_INPUTS = {'rupture_model', 'exposure', 'site_model',
                     'source_model', 'shakemap', 'gmfs', 'gsim_logic_tree',
-                    'source_model_logic_tree', 'hazard_curves', 'insurance',
+                    'source_model_logic_tree', 'hazard_curves',
+                    'insurance', 'reinsurance', 'ins_loss',
                     'sites', 'job_ini', 'multi_peril', 'taxonomy_mapping',
                     'fragility', 'consequence', 'reqv', 'input_zip',
                     'amplification',
@@ -850,6 +868,7 @@ class OqParam(valid.ParamSet):
     floating_y_step = valid.Param(valid.positivefloat, 0)
     ignore_encoding_errors = valid.Param(valid.boolean, False)
     ignore_master_seed = valid.Param(valid.boolean, False)
+    epsilon_star = valid.Param(valid.boolean, False)
     export_dir = valid.Param(valid.utf8, '.')
     exports = valid.Param(valid.export_formats, ())
     gmf_max_gb = valid.Param(valid.positivefloat, .1)
@@ -940,6 +959,7 @@ class OqParam(valid.ParamSet):
     sites_slice = valid.Param(valid.simple_slice, None)
     soil_intensities = valid.Param(valid.positivefloats, None)
     source_id = valid.Param(valid.namelist, [])
+    source_nodes = valid.Param(valid.namelist, [])
     spatial_correlation = valid.Param(valid.Choice('yes', 'no', 'full'), 'yes')
     specific_assets = valid.Param(valid.namelist, [])
     split_sources = valid.Param(valid.boolean, True)
@@ -947,7 +967,12 @@ class OqParam(valid.ParamSet):
     ebrisk_maxsize = valid.Param(valid.positivefloat, 2E10)  # used in ebrisk
     time_event = valid.Param(str, None)
     time_per_task = valid.Param(valid.positivefloat, 2000)
-    truncation_level = valid.Param(valid.NoneOr(valid.positivefloat), None)
+    total_losses = valid.Param(
+        valid.Choice('structural+nonstructural',
+                     'structural+contents',
+                     'nonstructural+contents',
+                     'structural+nonstructural+contents'), None)
+    truncation_level = valid.Param(valid.positivefloat, 99.)
     uniform_hazard_spectra = valid.Param(valid.boolean, False)
     vs30_tolerance = valid.Param(valid.positiveint, 0)
     width_of_mfd_bin = valid.Param(valid.positivefloat, None)
@@ -1044,9 +1069,9 @@ class OqParam(valid.ParamSet):
                 self.intensity_measure_types, [0])
             delattr(self, 'intensity_measure_types')
         if ('ps_grid_spacing' in names_vals and
+                float(names_vals['ps_grid_spacing']) and
                 'pointsource_distance' not in names_vals):
-            raise InvalidFile('%s: ps_grid_spacing requires setting a '
-                              'pointsource_distance!' % self.inputs['job_ini'])
+            self.pointsource_distance = dict(default=10.)
         if self.collapse_level >= 0:
             self.time_per_task = 1_000_000  # disable task_splitting
 
@@ -1323,7 +1348,8 @@ class OqParam(valid.ParamSet):
                     mini[imt] = 0
         if 'default' in mini:
             del mini['default']
-        return numpy.array([mini.get(imt) or 1E-10 for imt in self.imtls])
+        min_iml = numpy.array([mini.get(imt) or 1E-10 for imt in self.imtls])
+        return min_iml
 
     def levels_per_imt(self):
         """
@@ -1418,7 +1444,7 @@ class OqParam(valid.ParamSet):
         """
         Dictionary extended_loss_type -> extended_loss_type index
         """
-        return {lt: i for i, (lt, dt) in enumerate(self.loss_dt_list())}
+        return {lt: i for i, lt in enumerate(self.ext_loss_types)}
 
     @property
     def loss_types(self):
@@ -1432,6 +1458,19 @@ class OqParam(valid.ParamSet):
             names.append(lt)
         return names
 
+    @property
+    def ext_loss_types(self):
+        """
+        :returns: list of loss types + secondary loss types
+        """
+        etypes = self.loss_types
+        if self.total_losses:
+            etypes = self.loss_types + [self.total_losses]
+        if 'insurance' in self.inputs:
+            itypes = [lt + '_ins' for lt in self.inputs['insurance']]
+            etypes = self.loss_types + itypes
+        return etypes
+
     def loss_dt(self, dtype=F64):
         """
         :returns: a composite dtype based on the loss types including occupants
@@ -1579,13 +1618,22 @@ class OqParam(valid.ParamSet):
         return (self.calculation_mode in
                 'event_based_risk ebrisk event_based_damage')
 
+    def is_valid_disagg_by_src(self):
+        """
+        disagg_by_src can be set only if ps_grid_spacing = 0
+        """
+        if self.disagg_by_src:
+            return self.ps_grid_spacing == 0
+        return True
+
     def is_valid_shakemap(self):
         """
         hazard_calculation_id must be set if shakemap_id is set
         """
         if self.shakemap_uri:
             kind = self.shakemap_uri['kind']
-            sig = inspect.signature(get_array[kind])
+            get_array = getattr(shakemap.parsers, 'get_array_' + kind)
+            sig = inspect.signature(get_array)
             # parameters without default value
             params = [p.name for p in list(
                 sig.parameters.values()) if p.default is p.empty]
@@ -1608,15 +1656,6 @@ class OqParam(valid.ParamSet):
         else:
             return True
 
-    def is_valid_truncation_level_disaggregation(self):
-        """
-        Truncation level must be set for disaggregation calculations
-        """
-        if self.calculation_mode == 'disaggregation':
-            return self.truncation_level is not None
-        else:
-            return True
-
     def is_valid_geometry(self):
         """
         It is possible to infer the geometry only if exactly
@@ -1712,15 +1751,6 @@ class OqParam(valid.ParamSet):
             ('classical', 'disaggregation'))
         return not invalid
 
-    def is_valid_ps_grid_spacing(self):
-        """
-        `ps_grid_spacing` must be smaller than the `pointsource_distance`
-        """
-        if not self.ps_grid_spacing:
-            return True
-        return self.ps_grid_spacing <= calc.filters.getdefault(
-            self.pointsource_distance, 'default')
-
     def is_valid_soil_intensities(self):
         """
         soil_intensities must be defined only in classical calculations
@@ -1830,7 +1860,8 @@ class OqParam(valid.ParamSet):
     def check_source_model(self):
         if ('hazard_curves' in self.inputs or 'gmfs' in self.inputs or
                 'multi_peril' in self.inputs or 'rupture_model' in self.inputs
-                or 'scenario' in self.calculation_mode):
+                or 'scenario' in self.calculation_mode
+                or 'ins_loss' in self.inputs):
             return
         if ('source_model_logic_tree' not in self.inputs and
                 self.inputs['job_ini'] != '<in-memory>' and
