@@ -516,7 +516,8 @@ def get_site_collection(oqparam, h5=None):
     :param oqparam:
         an :class:`openquake.commonlib.oqvalidation.OqParam` instance
     """
-    if h5 and 'sitecol' in h5:
+    ss = oqparam.sites_slice  # can be None or (start, stop)
+    if h5 and 'sitecol' in h5 and not ss:
         return h5['sitecol']
     mesh = get_mesh(oqparam, h5)
     if mesh is None and oqparam.ground_motion_fields:
@@ -535,11 +536,14 @@ def get_site_collection(oqparam, h5=None):
             req_site_params.add('ampcode')
         if h5 and 'site_model' in h5:  # comes from a site_model.csv
             sm = h5['site_model'][:]
+        elif (not h5 and 'site_model' in oqparam.inputs and
+              'exposure' not in oqparam.inputs):
+            # tested in test_with_site_model
+            sm = get_site_model(oqparam)
         else:
             sm = oqparam
         sitecol = site.SiteCollection.from_points(
             mesh.lons, mesh.lats, mesh.depths, sm, req_site_params)
-    ss = oqparam.sites_slice  # can be None or (start, stop)
     if ss:
         if 'custom_site_id' not in sitecol.array.dtype.names:
             gh = sitecol.geohash(6)
@@ -561,6 +565,13 @@ def get_site_collection(oqparam, h5=None):
     if ('vs30' in sitecol.array.dtype.names and
             not numpy.isnan(sitecol.vs30).any()):
         assert sitecol.vs30.max() < 32767, sitecol.vs30.max()
+
+    # sanity check on the site parameters
+    for param in req_site_params:
+        dt = site.site_param_dt[param]
+        if dt is F64 and (sitecol.array[param] == 0.).all():
+            raise ValueError('The site parameter %s is always zero: please '
+                             'check the site model' % param)
     return sitecol
 
 
@@ -725,17 +736,21 @@ def _check_csm(csm, oqparam, h5):
 
     # build a smart SourceFilter
     try:
-        sitecol = get_site_collection(oqparam, h5)  # already stored
-    except Exception:  # missing sites.csv in test_case_1_ruptures
-        sitecol = None
+        sitecol = get_site_collection(oqparam, h5)
+    except ValueError:   # already stored (case_66)
+        sitecol = h5['sitecol']
     csm.sitecol = sitecol
-    if sitecol is None:
+    if sitecol is None:  # missing sites.csv (test_case_1_ruptures)
         return
     srcfilter = SourceFilter(sitecol, oqparam.maximum_distance)
     logging.info('Checking the sources bounding box')
     lons = []
     lats = []
     for src in srcs:
+        if hasattr(src, 'location'):
+            lons.append(src.location.x)
+            lats.append(src.location.y)
+            continue
         try:
             box = srcfilter.get_enlarged_box(src)
         except BBoxError as exc:
@@ -754,9 +769,6 @@ def _check_csm(csm, oqparam, h5):
         raise BBoxError(
             'The bounding box of the sources is larger than half '
             'the globe: %d degrees' % (bbox[2] - bbox[0]))
-    sids = sitecol.within_bbox(bbox)
-    if len(sids) == 0:
-        raise RuntimeError('All sources were discarded!?')
 
 
 def get_composite_source_model(oqparam, h5=None, branchID=None):
@@ -926,15 +938,15 @@ def get_sitecol_assetcol(oqparam, haz_sitecol=None, cost_types=()):
         for sid, assets in zip(sitecol.sids, assets_by):
             assets_by_site[sid] = assets
             num_assets += len(assets)
-        logging.info('Associated {:_d} assets to {:_d} sites'.format(
-            num_assets, len(sitecol)))
+        logging.info('Associated {:_d} assets to {:_d} sites'.
+                     format(num_assets, len(sitecol)))
     else:
         # asset sites and hazard sites are the same
         sitecol = haz_sitecol
         assets_by_site = exposure.assets_by_site
         discarded = []
-        logging.info('Read %d sites and %d assets from the exposure',
-                     len(sitecol), sum(len(a) for a in assets_by_site))
+        logging.info('Read {:_d} sites and {:_d} assets from the exposure'.
+                     format(len(sitecol), sum(len(a) for a in assets_by_site)))
 
     assetcol = asset.AssetCollection(
         exposure, assets_by_site, oqparam.time_event, oqparam.aggregate_by)
@@ -953,6 +965,11 @@ def get_sitecol_assetcol(oqparam, haz_sitecol=None, cost_types=()):
         # you can in other cases, typically with a grid which is mostly empty
         # (i.e. there are many hazard sites with no assets)
         assetcol.reduce_also(sitecol)
+    if 'custom_site_id' not in sitecol.array.dtype.names:
+        gh = sitecol.geohash(8)
+        if len(numpy.unique(gh)) < len(gh):
+            logging.error('geohashes are not unique')
+        sitecol.add_col('custom_site_id', 'S8', gh)
     return sitecol, assetcol, discarded
 
 
@@ -996,8 +1013,8 @@ def _taxonomy_mapping(filename, taxonomies):
     taxonomies = taxonomies[1:]  # strip '?'
     missing = set(taxonomies) - set(dic)
     if missing:
-        raise InvalidFile('The taxonomies %s are in the exposure but not in '
-                          'the taxonomy mapping %s' % (missing, filename))
+        raise InvalidFile('The taxonomy strings %s are in the exposure but not in '
+                          'the taxonomy mapping file %s' % (missing, filename))
     lst = [[("?", 1)]]
     for taxo in taxonomies:
         recs = dic[taxo]
