@@ -15,24 +15,30 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
+
+import re
 import copy
 import os.path
 import pickle
 import operator
 import logging
+import collections
 import gzip
 import zlib
 import numpy
 
 from openquake.baselib import parallel, general, hdf5
-from openquake.hazardlib import nrml, sourceconverter, InvalidFile, pmf, geo
-from openquake.hazardlib.contexts import basename
+from openquake.hazardlib import nrml, sourceconverter, InvalidFile, tom
+from openquake.hazardlib.contexts import ContextMaker, basename
 from openquake.hazardlib.calc.filters import magstr
 from openquake.hazardlib.lt import apply_uncertainties
+from openquake.hazardlib.geo.surface.kite_fault import kite_to_geom
 
 TWO16 = 2 ** 16  # 65,536
+TWO32 = 2 ** 32  # 4,294,967,296
 by_id = operator.attrgetter('source_id')
-CALC_TIME, NUM_SITES, EFF_RUPTURES, WEIGHT = 3, 4, 5, 6
+
+CALC_TIME, NUM_SITES, NUM_RUPTURES, WEIGHT, MUTEX = 3, 4, 5, 6, 7
 
 source_info_dt = numpy.dtype([
     ('source_id', hdf5.vstr),          # 0
@@ -40,12 +46,30 @@ source_info_dt = numpy.dtype([
     ('code', (numpy.string_, 1)),      # 2
     ('calc_time', numpy.float32),      # 3
     ('num_sites', numpy.uint32),       # 4
-    ('eff_ruptures', numpy.uint32),    # 5
+    ('num_ruptures', numpy.uint32),    # 5
     ('weight', numpy.float32),         # 6
-    ('trti', numpy.uint8),             # 7
+    ('mutex_weight', numpy.float64),   # 7
+    ('trti', numpy.uint8),             # 8
 ])
 
 
+
+def fragmentno(src):
+    "Postfix after :.; as an integer"
+    fragment = re.split('[:.;]', src.source_id, 1)[1]
+    return int(fragment.replace('.', '').replace(';', ''))
+
+
+def mutex_by_grp(src_groups):
+    """
+    :returns: a composite array with boolean fields src_mutex, rup_mutex
+    """
+    lst = []
+    for sg in src_groups:
+        lst.append((sg.src_interdep == 'mutex', sg.rup_interdep == 'mutex'))
+    return numpy.array(lst, [('src_mutex', bool), ('rup_mutex', bool)])
+
+
 def create_source_info(csm, h5):
     """
     Creates source_info, source_wkt, trt_smrs, toms
@@ -53,26 +77,32 @@ def create_source_info(csm, h5):
     data = {}  # src_id -> row
     wkts = []
     lens = []
-    for sg in csm.src_groups:
-        for src in sg:
-            srcid = basename(src)
-            trti = csm.full_lt.trti.get(src.tectonic_region_type, -1)
+    for srcid, srcs in general.groupby(
+            csm.get_sources(), basename).items():
+        src = srcs[0]
+        num_ruptures = sum(src.num_ruptures for src in srcs)
+        mutex = getattr(src, 'mutex_weight', 0)
+        trti = csm.full_lt.trti.get(src.tectonic_region_type, -1)
+        if src.code == b'p':
+            code = b'p'
+        else:
             code = csm.code.get(srcid, b'P')
-            lens.append(len(src.trt_smrs))
-            row = [srcid, src.grp_id, code, 0, 0, 0, trti, 0]
-            wkts.append(getattr(src, '_wkt', ''))
-            data[srcid] = row
-            src.id = len(data) - 1
+        lens.append(len(src.trt_smrs))
+        row = [srcid, src.grp_id, code, 0, 0, num_ruptures,
+               src.weight, mutex, trti]
+        wkts.append(getattr(src, '_wkt', ''))
+        data[srcid] = row
+        srcid = len(data) - 1
+        for src in srcs:
+            src.id = srcid
 
     logging.info('There are %d groups and %d sources with len(trt_smrs)=%.2f',
-                 len(csm.src_groups), sum(len(sg) for sg in csm.src_groups),
-                 numpy.mean(lens))
+                 len(csm.src_groups), len(data), numpy.mean(lens))
     csm.source_info = data  # src_id -> row
     num_srcs = len(csm.source_info)
     # avoid hdf5 damned bug by creating source_info in advance
     h5.create_dataset('source_info',  (num_srcs,), source_info_dt)
-    h5['source_info'].attrs['atomic'] = any(
-        grp.atomic for grp in csm.src_groups)
+    h5['mutex_by_grp'] = mutex_by_grp(csm.src_groups)
     h5['source_wkt'] = numpy.array(wkts, hdf5.vstr)
 
 
@@ -109,25 +139,6 @@ def _check_dupl_ids(src_groups):
                 first = False
 
 
-def collapse_nphc(src):
-    """
-    Collapse the nodal_plane_distribution and hypocenter_distribution.
-    """
-    if (hasattr(src, 'nodal_plane_distribution') and
-            hasattr(src, 'hypocenter_distribution')):
-        if len(src.nodal_plane_distribution.data) > 1:
-            ws, nps = zip(*src.nodal_plane_distribution.data)
-            strike = numpy.average([np.strike for np in nps], weights=ws)
-            dip = numpy.average([np.dip for np in nps], weights=ws)
-            rake = numpy.average([np.rake for np in nps], weights=ws)
-            val = geo.NodalPlane(strike, dip, rake)
-            src.nodal_plane_distribution = pmf.PMF([(1., val)])
-        if len(src.hypocenter_distribution.data) > 1:
-            ws, vals = zip(*src.hypocenter_distribution.data)
-            val = numpy.average(vals, weights=ws)
-            src.hypocenter_distribution = pmf.PMF([(1., val)])
-
-
 def get_csm(oq, full_lt, h5=None):
     """
     Build source models from the logic tree and to store
@@ -140,8 +151,8 @@ def get_csm(oq, full_lt, h5=None):
         oq.source_id,
         discard_trts=[s.strip() for s in oq.discard_trts.split(',')],
         floating_x_step=oq.floating_x_step,
-        floating_y_step=oq.floating_y_step)
-    classical = not oq.is_event_based()
+        floating_y_step=oq.floating_y_step,
+        source_nodes=oq.source_nodes)
     full_lt.ses_seed = oq.ses_seed
     logging.info('Reading the source model(s) in parallel')
 
@@ -157,7 +168,8 @@ def get_csm(oq, full_lt, h5=None):
                               h5=h5 if h5 else None).reduce()
     if len(smdict) > 1:  # really parallel
         parallel.Starmap.shutdown()  # save memory
-    fix_geometry_sections(smdict)
+    fix_geometry_sections(smdict, h5)
+    logging.info('Applying uncertainties')
     groups = _build_groups(full_lt, smdict)
 
     # checking the changes
@@ -165,18 +177,10 @@ def get_csm(oq, full_lt, h5=None):
     if changes:
         logging.info('Applied {:_d} changes to the composite source model'.
                      format(changes))
-
-    if 'reqv' in oq.inputs:
-        logging.warning('Using equivalent distance approximation and '
-                        'collapsing finite size parameters in point sources')
-        for group in groups:
-            for src in group:
-                collapse_nphc(src)
-
     return _get_csm(full_lt, groups)
 
 
-def fix_geometry_sections(smdict):
+def fix_geometry_sections(smdict, h5):
     """
     If there are MultiFaultSources, fix the sections according to the
     GeometryModels (if any).
@@ -201,16 +205,32 @@ def fix_geometry_sections(smdict):
         sections.update(gmod.sections)
     nrml.check_unique(
         sec_ids, 'section ID in files ' + ' '.join(gfiles))
-    sections = {sid: sections[sid] for sid in sorted(sections)}
+    s2i = {suid: i for i, suid in enumerate(sorted(sections))}
+    sections = [sections[suid] for suid in sorted(sections)]
+    for idx, sec in enumerate(sections):
+        sec.suid = idx
+    if h5 and sections:
+        h5.save_vlen('multi_fault_sections',
+                     [kite_to_geom(sec) for sec in sections])
 
     # fix the MultiFaultSources
+    section_idxs = []
     for smod in smodels:
         for sg in smod.src_groups:
             for src in sg:
                 if hasattr(src, 'set_sections'):
                     if not sections:
                         raise RuntimeError('Missing geometryModel files!')
-                    src.set_sections(sections)
+                    if h5:
+                        src.hdf5path = h5.filename
+                    src.rupture_idxs = [tuple(s2i[idx] for idx in idxs)
+                                        for idxs in src.rupture_idxs]
+                    for idxs in src.rupture_idxs:
+                        section_idxs.extend(idxs)
+    cnt = collections.Counter(section_idxs)
+    if cnt:
+        mean_counts = numpy.mean(list(cnt.values()))
+        logging.info('Section multiplicity = %.1f', mean_counts)
 
 
 def _groups_ids(smlt_dir, smdict, fnames):
@@ -351,7 +371,27 @@ class CompositeSourceModel:
             for src in sg:
                 src.grp_id = grp_id
                 if src.code != b'P':
-                    self.code[src.source_id] = src.code
+                    source_id = basename(src)
+                    self.code[source_id] = src.code
+
+    # used for debugging; assume PoissonTOM; use read_cmakers instead
+    def _get_cmakers(self, oq):
+        cmakers = []
+        trt_smrs = self.get_trt_smrs()
+        rlzs_by_gsim_list = self.full_lt.get_rlzs_by_gsim_list(trt_smrs)
+        trts = list(self.full_lt.gsim_lt.values)
+        num_eff_rlzs = len(self.full_lt.sm_rlzs)
+        start = 0
+        for grp_id, rlzs_by_gsim in enumerate(rlzs_by_gsim_list):
+            trti = trt_smrs[grp_id][0] // num_eff_rlzs
+            cmaker = ContextMaker(trts[trti], rlzs_by_gsim, oq)
+            cmaker.tom = tom.PoissonTOM(oq.investigation_time)
+            cmaker.trti = trti
+            cmaker.start = start
+            cmaker.grp_id = grp_id
+            start += len(rlzs_by_gsim)
+            cmakers.append(cmaker)
+        return cmakers
 
     def get_trt_smrs(self):
         """
@@ -401,12 +441,6 @@ class CompositeSourceModel:
             for src in sg:
                 if hasattr(src, 'mags'):  # MultiFaultSource
                     srcmags = {magstr(mag) for mag in src.mags}
-                    if hasattr(src, 'source_file'):  # UcerfSource
-                        grid_key = "/".join(["Grid", src.ukey["grid_key"]])
-                        # for instance Grid/FM0_0_MEANFS_MEANMSR_MeanRates
-                        with hdf5.File(src.source_file, "r") as h5:
-                            srcmags.update(magstr(mag) for mag in
-                                           h5[grid_key + "/Magnitude"][:])
                 elif hasattr(src, 'data'):  # nonparametric
                     srcmags = {magstr(item[0].mag) for item in src.data}
                 else:
@@ -434,14 +468,15 @@ class CompositeSourceModel:
         """
         Update (eff_ruptures, num_sites, calc_time) inside the source_info
         """
-        for src_id, nsites, nrupts, weight, ctimes in zip(
-                source_data['src_id'], source_data['nsites'],
-                source_data['nrupts'], source_data['weight'],
-                source_data['ctimes']):
-            row = self.source_info[src_id.split(':')[0]]
+        assert len(source_data) < TWO32, len(source_data)
+        for src_id, grp_id, nsites, weight, ctimes in zip(
+                source_data['src_id'], source_data['grp_id'],
+                source_data['nsites'],
+                source_data['weight'], source_data['ctimes']):
+            baseid = basename(src_id)
+            row = self.source_info[baseid]
             row[CALC_TIME] += ctimes
             row[WEIGHT] += weight
-            row[EFF_RUPTURES] += nrupts
             row[NUM_SITES] += nsites
 
     def count_ruptures(self):
@@ -453,6 +488,22 @@ class CompositeSourceModel:
             n += src.count_ruptures()
         return n
 
+    def fix_src_offset(self):
+        """
+        Set the src.offset field for each source
+        """
+        for srcs in general.groupby(self.get_sources(), basename).values():
+            offset = 0
+            if len(srcs) > 1:  # order by split number
+                srcs.sort(key=fragmentno)
+            for src in srcs:
+                src.offset = offset
+                offset += src.num_ruptures
+                if src.num_ruptures >= TWO32:
+                    raise ValueError(
+                        '%s contains more than 2**32 ruptures' % src)
+                # print(src, src.offset, offset)
+
     def get_max_weight(self, oq):  # used in preclassical
         """
         :param oq: an OqParam instance
@@ -460,7 +511,9 @@ class CompositeSourceModel:
         """
         srcs = self.get_sources()
         tot_weight = 0
+        nr = 0
         for src in srcs:
+            nr += src.num_ruptures
             tot_weight += src.weight
             if src.code == b'C' and src.num_ruptures > 20_000:
                 msg = ('{} is suspiciously large, containing {:_d} '
