@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2021 GEM Foundation
+# Copyright (C) 2014-2022 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -48,17 +48,13 @@ def _get_phi_al_regional(C, mag, vs30measured, rrup):
     """
     Returns intra-event (Phi) standard deviation (equation 24, page 1046)
     """
-    phi_al = np.ones((len(vs30measured)))
-    s1 = np.ones_like(phi_al) * C['s1e']
-    s2 = np.ones_like(phi_al) * C['s2e']
+    s1 = np.ones_like(mag) * C['s1e']
+    s2 = np.ones_like(mag) * C['s2e']
     s1[vs30measured] = C['s1m']
     s2[vs30measured] = C['s2m']
-    if mag < 4:
-        phi_al *= s1
-    elif mag <= 6:
-        phi_al *= s1 + (s2 - s1) / 2. * (mag - 4.)
-    else:
-        phi_al *= s2
+    phi_al = s1 + (s2 - s1) / 2. * (mag - 4.)
+    phi_al[mag < 4] = s1[mag < 4]
+    phi_al[mag >= 6] = s2[mag >= 6]
     return phi_al
 
 
@@ -71,7 +67,7 @@ def _get_phi_al_regional_JPN(C, mag, vs30measured, rrup):
     idx = rrup < 30
     phi_al[idx] *= C['s5']
 
-    idx = ((rrup <= 80) & (rrup >= 30.))
+    idx = (rrup <= 80.) & (rrup >= 30.)
     phi_al[idx] *= C['s5'] + (C['s6'] - C['s5']) / 50. * (rrup[idx] - 30.)
 
     idx = rrup > 80
@@ -85,33 +81,30 @@ def _get_basic_term(C, ctx):
     Compute and return basic form, see page 1030.
     """
     # Fictitious depth calculation
-    if ctx.mag > 5.:
-        c4m = C['c4']
-    elif ctx.mag > 4.:
-        c4m = C['c4'] - (C['c4']-1.) * (5. - ctx.mag)
-    else:
-        c4m = 1.
+    c4m = C['c4'] - (C['c4'] - 1.) * (5. - ctx.mag)
+    c4m[ctx.mag > 5.] = C['c4']
+    c4m[ctx.mag < 4] = 1.
     R = np.sqrt(ctx.rrup**2. + c4m**2.)
     # basic form
     base_term = C['a1'] * np.ones_like(ctx.rrup) + C['a17'] * ctx.rrup
     # equation 2 at page 1030
-    if ctx.mag >= C['m1']:
-        base_term += (C['a5'] * (ctx.mag - C['m1']) +
-                      C['a8'] * (8.5 - ctx.mag)**2. +
-                      (C['a2'] + C['a3'] * (ctx.mag - C['m1'])) *
-                      np.log(R))
-    elif ctx.mag >= CONSTS['m2']:
-        base_term += (C['a4'] * (ctx.mag - C['m1']) +
-                      C['a8'] * (8.5 - ctx.mag)**2. +
-                      (C['a2'] + C['a3'] * (ctx.mag - C['m1'])) *
-                      np.log(R))
-    else:
-        base_term += (C['a4'] * (CONSTS['m2'] - C['m1']) +
-                      C['a8'] * (8.5 - CONSTS['m2'])**2. +
-                      C['a6'] * (ctx.mag - CONSTS['m2']) +
-                      C['a7'] * (ctx.mag - CONSTS['m2'])**2. +
-                      (C['a2'] + C['a3'] * (CONSTS['m2'] - C['m1'])) *
-                      np.log(R))
+    after = ctx.mag >= C['m1']
+    within = (ctx.mag >= CONSTS['m2']) & (ctx.mag < C['m1'])
+    before = ctx.mag < CONSTS['m2']
+    base_term[after] += (C['a5'] * (ctx.mag[after] - C['m1']) +
+                         C['a8'] * (8.5 - ctx.mag[after])**2. +
+                         (C['a2'] + C['a3'] * (ctx.mag[after] - C['m1'])) *
+                         np.log(R[after]))
+    base_term[within] += (C['a4'] * (ctx.mag[within] - C['m1']) +
+                          C['a8'] * (8.5 - ctx.mag[within])**2. +
+                          (C['a2'] + C['a3'] * (ctx.mag[within] - C['m1'])) *
+                          np.log(R[within]))
+    base_term[before] += (C['a4'] * (CONSTS['m2'] - C['m1']) +
+                          C['a8'] * (8.5 - CONSTS['m2'])**2. +
+                          C['a6'] * (ctx.mag[before] - CONSTS['m2']) +
+                          C['a7'] * (ctx.mag[before] - CONSTS['m2'])**2. +
+                          (C['a2'] + C['a3'] * (CONSTS['m2'] - C['m1'])) *
+                          np.log(R[before]))
     return base_term
 
 
@@ -124,8 +117,8 @@ def _get_derivative(C, sa1180, vs30):
     c = C['c']
     b = C['b']
     idx = vs30 < C['vlin']
-    derAmp[idx] = (b * sa1180[idx] * (-1./(sa1180[idx]+c) +
-                   1./(sa1180[idx] + c*(vs30[idx]/C['vlin'])**n)))
+    derAmp[idx] = (b * sa1180[idx] * (-1. / (sa1180[idx] + c) +
+                   1. / (sa1180[idx] + c * (vs30[idx] / C['vlin'])**n)))
     return derAmp
 
 
@@ -137,54 +130,42 @@ def _get_faulting_style_term(C, ctx):
     # this implements equations 5 and 6 at page 1032. f7 is the
     # coefficient for reverse mechanisms while f8 is the correction
     # factor for normal ruptures
-    if ctx.mag > 5.0:
-        f7 = C['a11']
-        f8 = C['a12']
-    elif ctx.mag >= 4:
-        f7 = C['a11'] * (ctx.mag - 4.)
-        f8 = C['a12'] * (ctx.mag - 4.)
-    else:
-        f7 = 0.0
-        f8 = 0.0
+    f7 = C['a11'] * np.clip(ctx.mag - 4., 0., 1.)
+    f8 = C['a12'] * np.clip(ctx.mag - 4., 0., 1.)
     # ranges of rake values for each faulting mechanism are specified in
     # table 2, page 1031
-    return (f7 * float(ctx.rake > 30 and ctx.rake < 150) +
-            f8 * float(ctx.rake > -150 and ctx.rake < -30))
+    return (f7 * ((ctx.rake > 30) & (ctx.rake < 150)) +
+            f8 * ((ctx.rake > -150) & (ctx.rake < -30)))
 
 
 def _get_hanging_wall_term(C, ctx):
     """
     Compute and return hanging wall model term, see page 1038.
     """
-    if ctx.dip == 90.0:
-        return np.zeros_like(ctx.rx)
-    else:
-        Fhw = np.zeros_like(ctx.rx)
-        Fhw[ctx.rx > 0] = 1.
-        # Taper 1
-        T1 = _hw_taper1(ctx)
-        # Taper 2
-        T2 = _hw_taper2(ctx)
-        # Taper 3
-        T3 = _hw_taper3(ctx)
-        # Taper 4
-        T4 = _hw_taper4(ctx)
-        # Taper 5
-        T5 = _hw_taper5(ctx)
-        # Finally, compute the hanging wall term
-        return Fhw*C['a13']*T1*T2*T3*T4*T5
+    Fhw = np.zeros_like(ctx.rx)
+    Fhw[ctx.rx > 0] = 1.
+    # Taper 1
+    T1 = _hw_taper1(ctx)
+    # Taper 2
+    T2 = _hw_taper2(ctx)
+    # Taper 3
+    T3 = _hw_taper3(ctx)
+    # Taper 4
+    T4 = _hw_taper4(ctx)
+    # Taper 5
+    T5 = _hw_taper5(ctx)
+    Fhw[ctx.dip == 90.] = 0.
+    # Finally, compute the hanging wall term
+    return Fhw * C['a13'] * T1 * T2 * T3 * T4 * T5
 
 
 def _get_inter_event_std(C, mag, sa1180, vs30):
     """
     Returns inter event (tau) standard deviation (equation 25, page 1046)
     """
-    if mag < 5:
-        tau_al = C['s3']
-    elif mag <= 7:
-        tau_al = C['s3'] + (C['s4'] - C['s3']) / 2. * (mag - 5.)
-    else:
-        tau_al = C['s4']
+    tau_al = C['s3'] + (C['s4'] - C['s3']) / 2. * (mag - 5.)
+    tau_al[mag < 5.] = C['s3']
+    tau_al[mag >= 7.] = C['s4']
     tau_b = tau_al
     tau = tau_b * (1 + _get_derivative(C, sa1180, vs30))
     return tau
@@ -303,10 +284,7 @@ def _get_top_of_rupture_depth_term(C, imt, ctx):
     Compute and return top of rupture depth term. See paragraph
     'Depth-to-Top of Rupture Model', page 1042.
     """
-    if ctx.ztor >= 20.0:
-        return C['a15']
-    else:
-        return C['a15'] * ctx.ztor / 20.0
+    return C['a15'] * np.clip(ctx.ztor / 20.0, None, 1.)
 
 
 def _get_vs30star(vs30, imt):
@@ -347,23 +325,19 @@ def _get_z1pt0ref(region, vs30):
 
 def _hw_taper1(ctx):
     # Compute taper t1
-    T1 = np.ones_like(ctx.rx)
-    T1 *= 60./45. if ctx.dip <= 30. else (90.-ctx.dip)/45.0
+    T1 = (90.-ctx.dip) / 45.0
+    T1[ctx.dip <= 30.] = 60./45.
     return T1
 
 
 def _hw_taper2(ctx):
     # Compute taper t2 (eq 12 at page 1039) - a2hw set to 0.2 as
     # indicated at page 1041
-    T2 = np.zeros_like(ctx.rx)
     a2hw = 0.2
-    if ctx.mag > 6.5:
-        T2 += (1. + a2hw * (ctx.mag - 6.5))
-    elif ctx.mag > 5.5:
-        T2 += (1. + a2hw * (ctx.mag - 6.5) - (1. - a2hw) *
-               (ctx.mag - 6.5)**2)
-    else:
-        T2 *= 0.
+    T2 = (1. + a2hw * (ctx.mag - 6.5) - (1. - a2hw) *
+          (ctx.mag - 6.5)**2)
+    T2[ctx.mag > 6.5] = 1. + a2hw * (ctx.mag[ctx.mag > 6.5] - 6.5)
+    T2[ctx.mag <= 5.5] = 0.
     return T2
 
 
@@ -373,23 +347,19 @@ def _hw_taper3(ctx):
     T3 = np.zeros_like(ctx.rx)
     r1 = ctx.width * np.cos(np.radians(ctx.dip))
     r2 = 3. * r1
-    #
     idx = ctx.rx < r1
     T3[idx] = (np.ones_like(ctx.rx)[idx] * CONSTS['h1'] +
-               CONSTS['h2'] * (ctx.rx[idx] / r1) +
-               CONSTS['h3'] * (ctx.rx[idx] / r1)**2)
-    #
-    idx = ((ctx.rx >= r1) & (ctx.rx <= r2))
-    T3[idx] = 1. - (ctx.rx[idx] - r1) / (r2 - r1)
+               CONSTS['h2'] * (ctx.rx[idx] / r1[idx]) +
+               CONSTS['h3'] * (ctx.rx[idx] / r1[idx])**2)
+    idx = (ctx.rx >= r1) & (ctx.rx <= r2)
+    T3[idx] = 1. - (ctx.rx[idx] - r1[idx]) / (r2[idx] - r1[idx])
     return T3
 
 
 def _hw_taper4(ctx):
     # Compute taper t4 (eq. 14 at page 1040)
     T4 = np.zeros_like(ctx.rx)
-    #
-    if ctx.ztor <= 10.:
-        T4 += (1. - ctx.ztor**2. / 100.)
+    T4[ctx.ztor <= 10.] = 1. - ctx.ztor[ctx.ztor <= 10.]**2. / 100.
     return T4
 
 
@@ -398,11 +368,9 @@ def _hw_taper5(ctx):
     # suggestions provided at page 1040
     T5 = np.zeros_like(ctx.rx)
     ry1 = ctx.rx * np.tan(np.radians(20.))
-    #
     idx = (ctx.ry0 - ry1) <= 0.0
     T5[idx] = 1.
-    #
-    idx = (((ctx.ry0 - ry1) > 0.0) & ((ctx.ry0 - ry1) < 5.0))
+    idx = ((ctx.ry0 - ry1) > 0.0) & ((ctx.ry0 - ry1) < 5.0)
     T5[idx] = 1. - (ctx.ry0[idx] - ry1[idx]) / 5.0
     return T5
 
@@ -425,8 +393,7 @@ def _get_sa_at_1180(region, C, imt, ctx):
             _get_hanging_wall_term(C, ctx) +
             _get_top_of_rupture_depth_term(C, imt, ctx) +
             _get_soil_depth_term(region, C, fake_z1pt0, vs30_1180) +
-            _get_regional_term(region, C, imt, vs30_1180, ctx.rrup)
-            )
+            _get_regional_term(region, C, imt, vs30_1180, ctx.rrup))
 
 
 class AbrahamsonEtAl2014(GMPE):
@@ -476,7 +443,7 @@ class AbrahamsonEtAl2014(GMPE):
         self.region = kwargs.get('region')
         assert self.region in (None, 'CHN', 'JPN', 'TWN'), region
 
-    def compute(self, ctx, imts, mean, sig, tau, phi):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
@@ -504,8 +471,7 @@ class AbrahamsonEtAl2014(GMPE):
                        _get_top_of_rupture_depth_term(C, imt, ctx) +
                        _get_faulting_style_term(C, ctx) +
                        _get_soil_depth_term(self.region, C, ctx.z1pt0,
-                                            ctx.vs30)
-                       )
+                                            ctx.vs30))
 
             mean[m] += _get_regional_term(
                 self.region, C, imt, ctx.vs30, ctx.rrup)
