@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2013-2021 GEM Foundation
+# Copyright (C) 2013-2022 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -37,7 +37,7 @@ from openquake.hazardlib.gsim.kotha_2016 import KothaEtAl2016Turkey
 from openquake.hazardlib.gsim.chiou_youngs_2014 import ChiouYoungs2014
 
 
-def compute(self, ctx, imts, mean, sig, tau, phi):
+def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
     """
     Adjustments for Armenia
     """
