@@ -318,6 +318,53 @@ def get_mean_values(SJ, C, ctx, a1100=None):
             _get_anelastic_attenuation_term(C, ctx.rrup))
 
 
+def _update_ctx(gsim, ctx):
+    """
+    Use the ztor, width and hypo_depth formula to estimate
+    if the estimate attribute is set.
+    """
+    if gsim.estimate_ztor:
+        # Equation 4 and 5 of Chiou & Youngs 2014
+        ctx.ztor = np.where(
+            (ctx.rake > 30.) & (ctx.rake < 150.),
+            np.maximum(2.704-1.226 * np.maximum(ctx.mag-5.849, 0), 0)**2,
+            np.maximum(2.673-1.136 * np.maximum(ctx.mag-4.970, 0), 0)**2)
+
+    if gsim.estimate_width:
+        # width estimation requires Zbot
+        # where Zbot is the depth to the bottom of the seismogenic crust
+        if not hasattr(ctx, "zbot"):
+            raise KeyError('Zbot is required if width is unknown.')
+
+        # Equation 39 of Campbell & Bozorgnia 2014
+        mask = np.absolute(np.sin(np.radians(ctx.dip))) > 0
+        ctx.width = np.sqrt(10**((ctx.mag - 4.07) / 0.98))
+        ctx.width[mask] = np.minimum(
+            ctx.width[mask], (ctx.zbot[mask] - ctx.ztor[mask]) /
+            np.sin(np.radians(ctx.dip[mask])))
+
+    if gsim.estimate_hypo_depth:
+        # Equation 36 of Campbell & Bozorgnia 2014
+        fdz_m = np.where(
+            ctx.mag < 6.75, -4.317 + 0.984 * ctx.mag, 2.325)
+
+        # Equation 37 of Campbell & Bozorgnia 2014
+        fdz_d = np.where(
+            ctx.dip <= 40, 0.0445 * (ctx.dip - 40), 0)
+
+        # The depth to the bottom of the rupture plane
+        zbor = ctx.ztor + ctx.width * np.sin(np.radians(ctx.dip))
+
+        # Equation 35 of Campbell & Bozorgnia 2014
+        mask = zbor > ctx.ztor
+        dz = np.zeros_like(ctx.ztor)
+        dz[mask] = np.exp(
+            np.minimum(
+                fdz_m[mask] + fdz_d[mask],
+                np.log(0.9 * (zbor[mask] - ctx.ztor[mask]))))
+        ctx.hypo_depth = ctx.ztor + dz
+
+
 class CampbellBozorgnia2014(GMPE):
     """
     Implements NGA-West 2 GMPE developed by Kenneth W. Campbell and Yousef
@@ -356,12 +403,23 @@ class CampbellBozorgnia2014(GMPE):
 
     SJ = 0  # 1 for Japan
 
+    def __init__(self, **kwargs):
+        self.kwargs = kwargs
+        self.estimate_ztor = int(kwargs.get('estimate_ztor', 0))
+        self.estimate_width = int(kwargs.get('estimate_width', 0))
+        self.estimate_hypo_depth = int(kwargs.get('estimate_hypo_depth', 0))
+
+        if self.estimate_width:
+            # To estimate a width, the GMPE needs Zbot
+            self.REQUIRES_RUPTURE_PARAMETERS |= {"zbot"}
+
     def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
+        _update_ctx(self, ctx)
         C_PGA = self.COEFFS[PGA()]
         # Get mean and standard deviation of PGA on rock (Vs30 1100 m/s^2)
         pga1100 = np.exp(get_mean_values(self.SJ, C_PGA, ctx))
