@@ -402,6 +402,33 @@ def get_tau(C, mag):
     return C['tau1'] + (C['tau2'] - C['tau1']) / 1.5 * mag_test
 
 
+def get_mean_stddevs(name, C, ctx):
+    """
+    Return mean and standard deviation values
+    """
+    # Get ground motion on reference rock
+    ln_y_ref = get_ln_y_ref(name, C, ctx)
+    y_ref = np.exp(ln_y_ref)
+    # Get the site amplification
+    # Get basin depth
+    dz1pt0 = _get_centered_z1pt0(name, ctx)
+    # for Z1.0 = 0.0 no deep soil correction is applied
+    dz1pt0[ctx.z1pt0 <= 0.0] = 0.0
+    f_z1pt0 = get_basin_depth_term(name, C, dz1pt0)
+    # Get linear amplification term
+    f_lin = get_linear_site_term(name, C, ctx)
+    # Get nonlinear amplification term
+    f_nl, f_nl_scaling = get_nonlinear_site_term(C, ctx, y_ref)
+
+    # Add on the site amplification
+    mean = ln_y_ref + (f_lin + f_nl + f_z1pt0)
+    # Get standard deviations
+    sig, tau, phi = get_stddevs(
+        name, C, ctx, ctx.mag, y_ref, f_nl_scaling)
+
+    return mean, sig, tau, phi
+
+
 class ChiouYoungs2014(GMPE):
     """
     Implements GMPE developed by Brian S.-J. Chiou and Robert R. Youngs
@@ -450,26 +477,23 @@ class ChiouYoungs2014(GMPE):
         for spec of input and result values.
         """
         name = self.__class__.__name__
+        # reference to page 1144, PSA might need PGA value
+        pga_mean, pga_sig, pga_tau, pga_phi = get_mean_stddevs(name, self.COEFFS[PGA()], ctx)
         for m, imt in enumerate(imts):
-            C = self.COEFFS[imt]
-            # Get ground motion on reference rock
-            ln_y_ref = get_ln_y_ref(name, C, ctx)
-            y_ref = np.exp(ln_y_ref)
-            # Get the site amplification
-            # Get basin depth
-            dz1pt0 = _get_centered_z1pt0(name, ctx)
-            # for Z1.0 = 0.0 no deep soil correction is applied
-            dz1pt0[ctx.z1pt0 <= 0.0] = 0.0
-            f_z1pt0 = get_basin_depth_term(name, C, dz1pt0)
-            # Get linear amplification term
-            f_lin = get_linear_site_term(name, C, ctx)
-            # Get nonlinear amplification term
-            f_nl, f_nl_scaling = get_nonlinear_site_term(C, ctx, y_ref)
-            # Add on the site amplification
-            mean[m] = ln_y_ref + (f_lin + f_nl + f_z1pt0)
-            # Get standard deviations
-            sig[m], tau[m], phi[m] = get_stddevs(
-                name, C, ctx, ctx.mag, y_ref, f_nl_scaling)
+            if repr(imt) == "PGA":
+                mean[m] = pga_mean
+                sig[m], tau[m], phi[m] = pga_sig, pga_tau, pga_phi
+            else:
+                imt_mean, imt_sig, imt_tau, imt_phi = \
+                    get_mean_stddevs(name, self.COEFFS[imt], ctx)
+                # reference to page 1144
+                # Predicted PSA value at T ≤ 0.3s should be set equal to the value of PGA
+                # when it falls below the predicted PGA
+                mean[m] = np.where(imt_mean < pga_mean, pga_mean, imt_mean) \
+                    if repr(imt).startswith("SA") and imt.period <= 0.3 \
+                    else imt_mean
+
+                sig[m], tau[m], phi[m] = imt_sig, imt_tau, imt_phi
 
     #: Coefficient tables are constructed from values in tables 1 - 5
     COEFFS = CoeffsTable(sa_damping=5, table="""\
