@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (c) 2013-2021 GEM Foundation
+# Copyright (c) 2013-2022 GEM Foundation
 #
 # openquake is free software: you can redistribute it and/or modify it
 # under the terms of the gnu affero general public license as published
@@ -153,7 +153,7 @@ class HongGoda2007(GMPE):
     #: GMPE not tested against independent implementation
     non_verified = True
 
-    def compute(self, ctx, imts, mean, sig, tau, phi):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
