@@ -1,5 +1,5 @@
 # The Hazard Library
-# Copyright (C) 2012-2021 GEM Foundation
+# Copyright (C) 2012-2022 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -173,7 +173,7 @@ class NRCan15SiteTerm(GMPE):
             assert (self.gmpe.DEFINED_FOR_REFERENCE_VELOCITY >= 760 and
                     self.gmpe.DEFINED_FOR_REFERENCE_VELOCITY <= 800)
 
-    def compute(self, ctx, imts, mean, sig, tau, phi):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
