@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2021 GEM Foundation
+# Copyright (C) 2014-2022 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -70,7 +70,7 @@ def _get_magnitude_scaling_term(C, mag):
     Returns the magnitude scaling component of the model
     Equation 10, Page 63
     """
-    return C["c1"] + (C["c2"] * mag) + (C["c3"] * (mag ** 2.0))
+    return C["c1"] + C["c2"] * mag + C["c3"] * mag ** 2
 
 
 class RietbrockEtAl2013SelfSimilar(GMPE):
@@ -98,7 +98,7 @@ class RietbrockEtAl2013SelfSimilar(GMPE):
 
     #: Supported intensity measure component is the geometric mean of two
     #: horizontal components
-    DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.AVERAGE_HORIZONTAL
+    DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.GEOMETRIC_MEAN
 
     #: Supported standard deviation types are inter-event, intra-event and
     #: total
@@ -114,7 +114,7 @@ class RietbrockEtAl2013SelfSimilar(GMPE):
     #: Required distance measure is Rjb
     REQUIRES_DISTANCES = {'rjb'}
 
-    def compute(self, ctx, imts, mean, sig, tau, phi):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
