@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2021 GEM Foundation
+# Copyright (C) 2012-2022 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -19,7 +19,7 @@
 """
 Module exports :class:`SadighEtAl1997`.
 """
-import numpy
+import numpy as np
 
 from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
@@ -52,26 +52,26 @@ def get_mean_deep_soil(mag, rrup, is_reverse, C):
 
     Implements an equation from table 4.
     """
-    c1 = numpy.where(is_reverse,
+    c1 = np.where(is_reverse,
                      COEFFS_SOIL_IMT_INDEPENDENT['c1r'],
                      COEFFS_SOIL_IMT_INDEPENDENT['c1ss'])
     c2 = COEFFS_SOIL_IMT_INDEPENDENT['c2']
     c3 = COEFFS_SOIL_IMT_INDEPENDENT['c3']
-    c4 = numpy.where(mag <= NEAR_FIELD_SATURATION_MAG,
+    c4 = np.where(mag <= NEAR_FIELD_SATURATION_MAG,
                      COEFFS_SOIL_IMT_INDEPENDENT['c4lowmag'],
                      COEFFS_SOIL_IMT_INDEPENDENT['c4himag'])
-    c5 = numpy.where(mag <= NEAR_FIELD_SATURATION_MAG,
+    c5 = np.where(mag <= NEAR_FIELD_SATURATION_MAG,
                      COEFFS_SOIL_IMT_INDEPENDENT['c5lowmag'],
                      COEFFS_SOIL_IMT_INDEPENDENT['c5himag'])
-    c6 = numpy.where(is_reverse, C['c6r'], C['c6ss'])
+    c6 = np.where(is_reverse, C['c6r'], C['c6ss'])
     # clip mag if greater than 8.5. This is to avoid
     # ValueError: negative number cannot be raised to a fractional power
-    mag = numpy.clip(mag, None, 8.5)
+    mag = np.clip(mag, None, 8.5)
     return (c1 + c2 * mag + c6 + C['c7'] * ((8.5 - mag) ** 2.5)
-            - c3 * numpy.log(rrup + c4 * numpy.exp(c5 * mag)))
+            - c3 * np.log(rrup + c4 * np.exp(c5 * mag)))
 
 
-def get_mean_rock(mag, rrup, is_reverse, C):
+def get_mean_rock(mag, rrup, is_reverse, low_coeffs, hi_coeffs):
     """
     Calculate and return the mean intensity for rock ctx.
 
@@ -79,10 +79,15 @@ def get_mean_rock(mag, rrup, is_reverse, C):
     """
     # clip mag if greater than 8.5. This is to avoid
     # ValueError: negative number cannot be raised to a fractional power
-    mag = numpy.clip(mag, None, 8.5)
+    mag = np.clip(mag, None, 8.5)
+
+    # determine the coefficients to use depending on the mag
+    C = np.where(mag <= NEAR_FIELD_SATURATION_MAG, low_coeffs, hi_coeffs)
+
+    # compute the mean
     mean = (C['c1'] + C['c2'] * mag + C['c3'] * ((8.5 - mag) ** 2.5)
-            + C['c4'] * numpy.log(rrup + numpy.exp(C['c5'] + C['c6'] * mag))
-            + C['c7'] * numpy.log(rrup + 2))
+            + C['c4'] * np.log(rrup + np.exp(C['c5'] + C['c6'] * mag))
+            + C['c7'] * np.log(rrup + 2))
     # footnote in table 2 says that for reverse ruptures
     # the mean amplitude value should be multiplied by 1.2
     mean[is_reverse] += 0.1823215567939546  # == log(1.2)
@@ -95,9 +100,8 @@ def get_stddev_rock(mag, C):
 
     Implements formulae from table 3.
     """
-    return numpy.where(mag > C['maxmag'],
-                       C['maxsigma'],
-                       C['sigma0'] + C['magfactor'] * mag)
+    return np.where(
+        mag > C['maxmag'], C['maxsigma'], C['sigma0'] + C['magfactor'] * mag)
 
 
 def get_stddev_deep_soil(mag, C):
@@ -108,7 +112,7 @@ def get_stddev_deep_soil(mag, C):
     """
     # footnote from table 4 says that stderr for magnitudes over 7
     # is equal to one of magnitude 7.
-    return C['sigma0'] + C['magfactor'] * numpy.clip(mag, None, 7)
+    return C['sigma0'] + C['magfactor'] * np.clip(mag, None, 7)
 
 
 class SadighEtAl1997(GMPE):
@@ -128,9 +132,9 @@ class SadighEtAl1997(GMPE):
 
     #: Supported intensity measure component is the geometric mean of
     #: two : horizontal components
-    #: :attr:`~openquake.hazardlib.const.IMC.AVERAGE_HORIZONTAL`, : see
+    #: :attr:`~openquake.hazardlib.const.IMC.GEOMETRIC_MEAN`, : see
     #: page 180.
-    DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.AVERAGE_HORIZONTAL
+    DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.GEOMETRIC_MEAN
 
     #: Supported standard deviation type is only total, see table 3.
     DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
@@ -145,7 +149,7 @@ class SadighEtAl1997(GMPE):
     #: Required distance measure is RRup (eq. 1).
     REQUIRES_DISTANCES = {'rrup'}
 
-    def compute(self, ctx, imts, mean, sig, tau, phi):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
         <.base.GroundShakingIntensityModel.compute>`
@@ -153,27 +157,27 @@ class SadighEtAl1997(GMPE):
         """
         # GMPE differentiates strike-slip, reverse and normal ruptures,
         # but combines normal and strike-slip into one category. See page 180.
-        is_reverse = (45 <= ctx.rake <= 135)
-        [rocks_i] = (ctx.vs30 > ROCK_VS30).nonzero()
-        [soils_i] = (ctx.vs30 <= ROCK_VS30).nonzero()
+        reverse = (45 <= ctx.rake) & (ctx.rake <= 135)
+        is_rock = ctx.vs30 > ROCK_VS30
+        is_soil = ctx.vs30 <= ROCK_VS30
+        mag_r = ctx.mag[is_rock]
+        mag_s = ctx.mag[is_soil]
+        rrup_r = ctx.rrup[is_rock]
+        rrup_s = ctx.rrup[is_soil]
+        reverse_r = reverse[is_rock]
+        reverse_s = reverse[is_soil]
         for m, imt in enumerate(imts):
-            if len(rocks_i):
-                rrup = ctx.rrup.take(rocks_i)
-                if ctx.mag <= NEAR_FIELD_SATURATION_MAG:
-                    C = self.COEFFS_ROCK_LOWMAG[imt]
-                else:
-                    C = self.COEFFS_ROCK_HIMAG[imt]
-                mean_rock = get_mean_rock(ctx.mag, rrup, is_reverse, C)
-                mean[m, rocks_i] = mean_rock
-                sig[m, rocks_i] = get_stddev_rock(
-                    ctx.mag, self.COEFFS_ROCK_STDDERR[imt])
-            if len(soils_i):
-                rrup = ctx.rrup.take(soils_i)
-                mean_soil = get_mean_deep_soil(
-                    ctx.mag, rrup, is_reverse, self.COEFFS_SOIL[imt])
-                mean[m, soils_i] = mean_soil
-                sig[m, soils_i] = get_stddev_deep_soil(
-                    ctx.mag, self.COEFFS_SOIL[imt])
+            mean[m, is_rock] = get_mean_rock(
+                mag_r, rrup_r, reverse_r,
+                self.COEFFS_ROCK_LOWMAG[imt],
+                self.COEFFS_ROCK_HIMAG[imt])
+            sig[m, is_rock] = get_stddev_rock(
+                mag_r, self.COEFFS_ROCK_STDDERR[imt])
+
+            mean[m, is_soil] = get_mean_deep_soil(
+                mag_s, rrup_s, reverse_s, self.COEFFS_SOIL[imt])
+            sig[m, is_soil] = get_stddev_deep_soil(
+                mag_s, self.COEFFS_SOIL[imt])
 
     #: Coefficients tables for rock ctx (table 2), for magnitude
     #: values of :attr:`NEAR_FIELD_SATURATION_MAG` and below. Damping
