@@ -21,8 +21,8 @@ import itertools
 import collections
 import numpy
 
-from openquake.baselib.general import CallableDict, BASE94
-from openquake.hazardlib import geo, source as ohs
+from openquake.baselib.general import CallableDict, BASE183
+from openquake.hazardlib import geo
 from openquake.hazardlib.sourceconverter import (
     split_coords_2d, split_coords_3d)
 from openquake.hazardlib import valid
@@ -94,8 +94,7 @@ def trucMFDFromSlip_absolute(utype, node, filename):
 
 @parse_uncertainty.add('setMSRAbsolute')
 def setMSR_absolute(utype, node, filename):
-    tmps = valid.mag_scale_rel(node.text)
-    return valid.SCALEREL[tmps]()
+    return valid.mag_scale_rel(node.text)
 
 
 @parse_uncertainty.add('simpleFaultGeometryAbsolute')
@@ -488,6 +487,10 @@ class Branch(object):
         self.value = value
         self.bset = None
 
+    @property
+    def id(self):
+        return self.branch_id if len(self.branch_id) == 1 else self.short_id
+
     def is_leaf(self):
         """
         :returns: True if the branch has no branchset or has a dummy branchset
@@ -605,11 +608,11 @@ class BranchSet(object):
             branches) and list of path's :class:`Branch` objects. Total sum
             of all paths' weights is 1.0
         """
-        for path in self._enumerate_paths([]):
+        for path_branch in self._enumerate_paths([]):
             flat_path = []
             weight = 1.0
-            while path:
-                path, branch = path
+            while path_branch:
+                path_branch, branch = path_branch
                 weight *= branch.weight
                 flat_path.append(branch)
             yield weight, flat_path[::-1]
@@ -627,15 +630,15 @@ class BranchSet(object):
         else:
             branches = self.branches
         for branch in branches:
-            path = [prefix_path, branch]
-            if branch.bset is not None:
-                yield from branch.bset._enumerate_paths(path)
+            path_branch = [prefix_path, branch]
+            if branch.bset is not None:  # dummies can be branchpoints
+                yield from branch.bset._enumerate_paths(path_branch)
             else:
-                yield path
+                yield path_branch
 
     def __getitem__(self, branch_id):
         """
-        Return :class:`Branch` object belonging to this branch set with id
+        Return :class:`Branch` object belonging to this branchset with id
         equal to ``branch_id``.
         """
         for branch in self.branches:
@@ -673,12 +676,24 @@ class BranchSet(object):
         bset = self
         while ltpath:
             brid, ltpath = ltpath[0], ltpath[1:]
-            pairs.append((bset, bset[brid].value))
-            bset = bset[brid].bset
-            if bset is None:
+            br = bset[brid]
+            pairs.append((bset, br.value))
+            if br.is_leaf():
                 break
+            else:
+                bset = br.bset
         return pairs
 
+    def to_list(self):
+        """
+        :returns: a literal list describing the branchset
+        """
+        atb = self.filters.get("applyToBranches", [])
+        lst = [self.uncertainty_type, atb]
+        for br in self.branches:
+            lst.append([br.branch_id, '...', br.weight])
+        return lst
+
     def __len__(self):
         return len(self.branches)
 
@@ -689,6 +704,17 @@ class BranchSet(object):
         return '<%s(%d)>' % (self.uncertainty_type, len(self))
 
 
+# NB: this function cannot be used with monster logic trees like the one for
+# South Africa (ZAF), since it is too slow; the engine use a trick instead
+def count_paths(branches):
+    """
+    :param branches: a list of branches (endpoints or nodes)
+    :returns: the number of paths in the branchset (slow)
+    """
+    return sum(1 if br.bset is None else count_paths(br.bset.branches)
+               for br in branches)
+
+
 dummy_counter = itertools.count(1)
 
 
@@ -725,23 +751,17 @@ class Realization(object):
             '~'.join(self.lt_path), self.weight, samples)
 
 
-def attach_to_branches(branchsets):
-    """
-    Attach branchsets to branches depending on the applyToBranches
-    attribute. Also attaches dummy branchsets to dummy branches.
-    """
-    previous_branches = branchsets[0].branches
-    for i, bset in enumerate(branchsets[1:]):
-        prev_ids = ' '.join(pb.branch_id for pb in previous_branches)
-        atb = bset.filters.get('applyToBranches') or prev_ids
-        dummies = []
-        for branch in previous_branches:
-            if branch.branch_id in atb:
-                branch.bset = bset
-            else:
-                branch.bset = dummy = dummy_branchset()
-                dummies.append(dummy.branches[0])
-        previous_branches = bset.branches + dummies
+def add_path(bset, bsno, brno, num_prev, tot, paths):
+    for br in bset.branches:
+        br.short_id = BASE183[brno]
+        path = ['*'] * tot
+        path[bsno] = br.id
+        paths.append(''.join(path))
+        brno += 1
+    if 'applyToBranches' not in bset.filters or len(
+            bset.filters['applyToBranches']) == num_prev:
+        return 0
+    return brno
 
 
 class CompositeLogicTree(object):
@@ -751,22 +771,49 @@ class CompositeLogicTree(object):
     """
     def __init__(self, branchsets):
         self.branchsets = branchsets
-        attach_to_branches(branchsets)
-        nb = len(branchsets)
+        self.basepaths = self._attach_to_branches()
+
+    def _attach_to_branches(self):
+        # attach branchsets to branches depending on the applyToBranches
+        # attribute; also attaches dummy branchsets to dummy branches.
         paths = []
-        for bsno, bset in enumerate(branchsets):
-            for brno, br in enumerate(bset.branches):
-                path = ['*'] * nb
-                path[bsno] = br.short_id = BASE94[brno]
-                paths.append(''.join(path))
-        self.basepaths = paths
+        nb = len(self.branchsets)
+        brno = add_path(self.branchsets[0], 0, 0, 0, nb, paths)
+        previous_branches = self.branchsets[0].branches
+        branchdic = {br.branch_id: br for br in previous_branches}
+        for i, bset in enumerate(self.branchsets[1:]):
+            for br in bset.branches:
+                if br.branch_id != '.' and br.branch_id in branchdic:
+                    raise NameError('The branch ID %s is duplicated'
+                                    % br.branch_id)
+                branchdic[br.branch_id] = br
+            dummies = []
+            prev_ids = [pb.branch_id for pb in previous_branches]
+            app2brs = list(bset.filters.get('applyToBranches', '')) or prev_ids
+            if app2brs != prev_ids:
+                for branch_id in app2brs:
+                    # NB: if branch_id has already a branchset it is overridden
+                    branchdic[branch_id].bset = bset
+                for brid in prev_ids:
+                    br = branchdic[brid]
+                    if brid not in app2brs:
+                        br.bset = dummy = dummy_branchset()
+                        [dummybranch] = dummy.branches
+                        branchdic[dummybranch.branch_id] = dummybranch
+                        dummies.append(dummybranch)
+            else:  # apply to all previous branches
+                for branch in previous_branches:
+                    branch.bset = bset
+            brno = add_path(bset, i+1, brno, len(previous_branches), nb, paths)
+            previous_branches = bset.branches + dummies
+        return paths
 
     def __iter__(self):
         nb = len(self.branchsets)
         ordinal = 0
         for weight, branches in self.branchsets[0].enumerate_paths():
             value = [br.value for br in branches]
-            lt_path = ''.join(branch.short_id for branch in branches)
+            lt_path = ''.join(branch.id for branch in branches)
             yield Realization(value, weight, ordinal, lt_path.ljust(nb, '.'))
             ordinal += 1
 
@@ -775,3 +822,29 @@ class CompositeLogicTree(object):
 
     def __repr__(self):
         return '<%s>' % self.branchsets
+
+
+def build(*bslists):
+    """
+    :param bslists: a list of lists describing branchsets
+    :returns: a `CompositeLogicTree` instance
+
+    >>> lt = build(['sourceModel', '',
+    ...              ['A', 'common1', 0.6],
+    ...              ['B', 'common2', 0.4]],
+    ...           ['extendModel', '',
+    ...              ['C', 'extra1', 0.6],
+    ...              ['D', 'extra2', 0.2],
+    ...              ['E', 'extra2', 0.2]])
+    >>> lt.get_all_paths()
+    ['AC', 'AD', 'AE', 'BC', 'BD', 'BE']
+    """
+    bsets = []
+    for i, (utype, applyto, *brlists) in enumerate(bslists):
+        branches = []
+        for brid, value, weight in brlists:
+            branches.append(Branch('bs%02d' % i, brid, weight, value))
+        bset = BranchSet(utype, i, dict(applyToBranches=applyto))
+        bset.branches = branches
+        bsets.append(bset)
+    return CompositeLogicTree(bsets)
