@@ -42,9 +42,15 @@ class BaseASR(metaclass=abc.ABCMeta):
             from -180 to 180.
         """
 
+    def __str__(self):
+        """
+        Returns a TOML representation of the instance
+        """
+        return self.__class__.__name__
+
     def __repr__(self):
         """
-        Returns the name of the class
+        Returns the name of the class in angular brackets
         """
         return "<%s>" % self.__class__.__name__
 
@@ -92,6 +98,12 @@ class BaseMSR(metaclass=abc.ABCMeta):
         """
         return self.__class__ is other.__class__
 
+    def __str__(self):
+        """
+        Returns a TOML representation of the instance
+        """
+        return self.__class__.__name__
+
     def __repr__(self):
         """
         Returns the name of the class
