@@ -182,13 +182,13 @@ class ComplexFaultSource(ParametricSeismicSource):
         Uses :func:`_float_ruptures` for finding possible rupture locations
         on the whole fault surface.
         """
-        slc = kwargs.get('slc', slice(None))
+        step = kwargs.get('step', 1)
         whole_fault_surface = ComplexFaultSurface.from_fault_data(
             self.edges, self.rupture_mesh_spacing)
         whole_fault_mesh = whole_fault_surface.mesh
         cell_center, cell_length, cell_width, cell_area = (
             whole_fault_mesh.get_cell_dimensions())
-        for mag, mag_occ_rate in self.get_annual_occurrence_rates():
+        for mag, mag_occ_rate in self.get_annual_occurrence_rates()[::step]:
             # min_mag is inside get_annual_occurrence_rates
             if mag_occ_rate == 0:
                 continue
@@ -199,7 +199,7 @@ class ComplexFaultSource(ParametricSeismicSource):
             rupture_slices = _float_ruptures(
                 rupture_area, rupture_length, cell_area, cell_length)
             occurrence_rate = mag_occ_rate / float(len(rupture_slices))
-            for rupture_slice in rupture_slices[slc]:
+            for rupture_slice in rupture_slices[::step**2]:
                 mesh = whole_fault_mesh[rupture_slice]
                 # XXX: use surface centroid as rupture's hypocenter
                 # XXX: instead of point with middle index
@@ -215,12 +215,6 @@ class ComplexFaultSource(ParametricSeismicSource):
                 rup.mag_occ_rate = mag_occ_rate
                 yield rup
 
-    def few_ruptures(self):
-        """
-        Fast version of iter_ruptures used in estimate_weight
-        """
-        yield from self.iter_ruptures(slc=slice(None, None, 25))
-
     def count_ruptures(self):
         """
         See :meth:
