@@ -133,14 +133,14 @@ class KiteFaultSource(ParametricSeismicSource):
         msr = self.magnitude_scaling_relationship
         tom = self.temporal_occurrence_model
         surface = self.surface
-        slc = kwargs.get('slc', slice(None))
-        for mag, mag_occ_rate in self.get_annual_occurrence_rates():
+        step = kwargs.get('step', 1)
+        for mag, mag_occ_rate in self.get_annual_occurrence_rates()[::step]:
 
             # Compute the area, length and width of the ruptures
             area = msr.get_median_area(mag=mag, rake=self.rake)
-            lng, wdt = get_discrete_dimensions(area, self.rupture_mesh_spacing,
-                                               self.rupture_aspect_ratio,
-                                               self.profiles_sampling)
+            lng, wdt = get_discrete_dimensions(
+                area, self.rupture_mesh_spacing,
+                self.rupture_aspect_ratio, self.profiles_sampling)
 
             # Get the number of nodes along the strike and dip. Note that
             # len and wdt should be both multiples of the sampling distances
@@ -167,7 +167,6 @@ class KiteFaultSource(ParametricSeismicSource):
             # Get the geometry of all the ruptures that the fault surface
             # accommodates
             ruptures = []
-
             for rup in self._get_ruptures(surface.mesh, rup_len, rup_wid,
                                           f_strike=fstrike, f_dip=fdip):
                 ruptures.append(rup)
@@ -176,18 +175,12 @@ class KiteFaultSource(ParametricSeismicSource):
             occurrence_rate = mag_occ_rate / len(ruptures)
 
             # Rupture generator
-            for rup in ruptures[slc]:
+            for rup in ruptures[::step]:
                 hypocenter = rup[0].get_center()
                 # Yield an instance of a ParametricProbabilisticRupture
                 yield ppr(mag, self.rake, self.tectonic_region_type,
                           hypocenter, rup[0], occurrence_rate, tom)
 
-    def few_ruptures(self):
-        """
-        Fast version of iter_ruptures used in estimate_weight
-        """
-        yield from self.iter_ruptures(slc=slice(None, None, 25))
-
     def _get_ruptures(self, omsh, rup_s, rup_d, f_strike=1, f_dip=1):
         """
         Returns all the ruptures admitted by a given geometry i.e. number of
@@ -252,12 +245,11 @@ class KiteFaultSource(ParametricSeismicSource):
                                omsh.depths[j:j + rup_d, i:i + rup_s])
                     yield (KiteSurface(msh), j, i)
 
-    # TODO
     def get_fault_surface_area(self) -> float:
         """
         Returns the area of the fault surface
         """
-        pass
+        raise NotImplementedError
 
     def __iter__(self):
         """
