@@ -18,42 +18,43 @@
 """
 Utilities to compute mean and quantile curves
 """
+import math
 import numpy
 import pandas
 from scipy.stats import norm
-from scipy.special import ndtr
 from openquake.baselib.general import AccumDict, agg_probs
+from openquake.baselib.performance import compile
+try:
+    import numba
+    SQRT05 = math.sqrt(0.5)
+
+    @numba.vectorize("float64(float64)")
+    def ndtr(z):
+        return 0.5 * (1.0 + math.erf(z * SQRT05))
+except ImportError:
+    from scipy.special import ndtr
 
 
+@compile(["float64[:,:](float64, float64[:,:])",
+          "float64[:](float64, float64[:])"])
 def _truncnorm_sf(truncation_level, values):
-    """
-    Survival function for truncated normal distribution.
-
-    Assumes zero mean, standard deviation equal to one and symmetric
-    truncation.
-
-    :param truncation_level:
-        Positive float number representing the truncation on both sides
-        around the mean, in units of sigma, or None, for non-truncation
-    :param values:
-        Numpy array of values as input to a survival function for the given
-        distribution.
-    :returns:
-        Numpy array of survival function results in a range between 0 and 1.
-
-    >>> from scipy.stats import truncnorm
-    >>> truncnorm(-3, 3).sf(0.12345) == _truncnorm_sf(3, 0.12345)
-    True
-    >>> from scipy.stats import norm
-    >>> norm.sf(0.12345) == _truncnorm_sf(None, 0.12345)
-    True
-    """
-    if truncation_level == 0:
+    # Fast survival function for truncated normal distribution.
+    # Assumes zero mean, standard deviation equal to one and symmetric
+    # truncation. It is faster than using scipy.stats.truncnorm.sf:
+    #
+    # _truncnorm_sf(sig, x) == truncnorm(-sig, sig).sf(x)
+    #
+    # :param truncation_level:
+    #     Positive float number representing the truncation on both sides
+    #     around the mean, in units of sigma, or None, for non-truncation
+    # :param values:
+    #     Numpy array of values as input to a survival function for the given
+    #     distribution.
+    # :returns:
+    #     Numpy array of survival function results in a range between 0 and 1.
+    if truncation_level == 0.:
         return values
 
-    if truncation_level is None:
-        return ndtr(- values)
-
     # notation from http://en.wikipedia.org/wiki/Truncated_normal_distribution.
     # given that mu = 0 and sigma = 1, we have alpha = a and beta = b.
 
@@ -68,7 +69,7 @@ def _truncnorm_sf(truncation_level, values):
 
     # calculate Z as ``Z = CDF(b) - CDF(a)``, here we assume that
     # ``CDF(a) == CDF(- truncation_level) == 1 - CDF(b)``
-    z = phi_b * 2 - 1
+    z = phi_b * 2. - 1.
 
     # calculate the result of survival function of ``values``,
     # and restrict it to the interval where probability is defined --
@@ -78,7 +79,7 @@ def _truncnorm_sf(truncation_level, values):
     # ``SF(x) = (Z - CDF(x) + CDF(a)) / Z``,
     # ``SF(x) = (CDF(b) - CDF(a) - CDF(x) + CDF(a)) / Z``,
     # ``SF(x) = (CDF(b) - CDF(x)) / Z``.
-    return ((phi_b - ndtr(values)) / z).clip(0.0, 1.0)
+    return ((phi_b - ndtr(values)) / z).clip(0., 1.)
 
 
 def norm_cdf(x, a, s):
@@ -361,23 +362,23 @@ def apply_stat(f, arraylist, *extra, **kw):
         return f(arraylist, *extra, **kw)
 
 
-def set_rlzs_stats(dstore, prefix, **attrs):
+def set_rlzs_stats(dstore, name, **attrs):
     """
     :param dstore: a DataStore object
-    :param prefix: dataset prefix, assume <prefix>-rlzs is already stored
+    :param name: dataset name of kind <prefix>-rlzs
     """
-    arrayNR = dstore[prefix + '-rlzs'][()]
+    arrayNR = dstore[name][()]
     R = arrayNR.shape[1]
     pairs = list(attrs.items())
     pairs.insert(1, ('rlz', numpy.arange(R)))
-    dstore.set_shape_descr(prefix + '-rlzs', **dict(pairs))
+    dstore.set_shape_descr(name, **dict(pairs))
     if R > 1:
         stats = dstore['oqparam'].hazard_stats()
         if not stats:
             return
         statnames, statfuncs = zip(*stats.items())
         weights = dstore['weights'][()]
-        name = prefix + '-stats'
+        name = name.replace('-rlzs', '-stats')
         dstore[name] = compute_stats2(arrayNR, statfuncs, weights)
         pairs = list(attrs.items())
         pairs.insert(1, ('stat', statnames))
