@@ -20,7 +20,7 @@
 Module :mod:`openquake.hazardlib.tom` contains implementations of probability
 density functions for earthquake temporal occurrence modeling.
 """
-import abc
+import toml
 import numpy
 import scipy.stats
 from openquake.baselib.performance import compile
@@ -29,7 +29,7 @@ registry = {}
 F64 = numpy.float64
 
 
-class BaseTOM(metaclass=abc.ABCMeta):
+class BaseTOM(object):
     """
     Base class for temporal occurrence model.
 
@@ -42,28 +42,26 @@ class BaseTOM(metaclass=abc.ABCMeta):
     def __init_subclass__(cls):
         registry[cls.__name__] = cls
 
-    def __init__(self, time_span, occurrence_rate=None):
+    def __init__(self, time_span):
         if time_span <= 0:
             raise ValueError('time_span must be positive')
         self.time_span = time_span
-        self.occurrence_rate = occurrence_rate
 
-    @abc.abstractmethod
     def get_probability_one_or_more_occurrences(self):
         """
         Calculate and return the probability of event to happen one or more
         times within the time range defined by constructor's ``time_span``
         parameter value.
         """
+        raise NotImplementedError
 
-    @abc.abstractmethod
     def get_probability_n_occurrences(self):
         """
         Calculate the probability of occurrence of a number of events in the
         constructor's ``time_span``.
         """
+        raise NotImplementedError
 
-    @abc.abstractmethod
     def sample_number_of_occurrences(self, seeds=None):
         """
         Draw a random sample from the distribution and return a number
@@ -74,7 +72,6 @@ class BaseTOM(metaclass=abc.ABCMeta):
         should be set outside of this method.
         """
 
-    @abc.abstractmethod
     def get_probability_no_exceedance(self):
         """
         Compute and return, for a number of ground motion levels and sites,
@@ -84,13 +81,16 @@ class BaseTOM(metaclass=abc.ABCMeta):
         exceedance in the time window specified by the ``time_span`` parameter
         given in the constructor.
         """
+        raise NotImplementedError
+
+    def __str__(self):
+        return toml.dumps({self.__class__.__name__: self.__dict__})
 
 
 class FatedTOM(BaseTOM):
 
-    def __init__(self, time_span, occurrence_rate=None):
+    def __init__(self, time_span):
         self.time_span = time_span
-        self.occurrence_rate = occurrence_rate
 
     def get_probability_one_or_more_occurrences(self, occurrence_rate):
         return 1
@@ -188,93 +188,198 @@ class PoissonTOM(BaseTOM):
         return numpy.exp(- occurrence_rate * self.time_span * poes)
 
 
-# use in calc.disagg
-def get_probability_no_exceedance_rup(rup, poes, tom):
+class ClusterPoissonTOM(PoissonTOM):
     """
-    Compute and return the probability that in the time span for which the
-    rupture is defined, the rupture itself never generates a ground motion
-    value higher than a given level at a given site.
-
-    Such calculation is performed starting from the conditional probability
-    that an occurrence of the current rupture is producing a ground motion
-    value higher than the level of interest at the site of interest.
-    The actual formula used for such calculation depends on the temporal
-    occurrence model the rupture is associated with.
-    The calculation can be performed for multiple intensity measure levels
-    and multiple sites in a vectorized fashion.
-
-    :param rup:
-        an object with a scalar attribute .occurrence_rate
-    :param poes:
-        array of shape (n, L, G) containing conditional probabilities that a
-        rupture occurrence causes a ground shaking value exceeding a
-        ground motion level at a site. First dimension represent sites,
-        second dimension intensity measure levels. ``poes`` can be obtained
-        calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`
-    :param tom:
-        temporal occurrence model (not used if the rupture is nonparametric)
+    Poissonian temporal occurrence model with an occurrence rate
     """
-    if numpy.isnan(rup.occurrence_rate):  # nonparametric
-        pnes = numpy.zeros_like(poes)
-        set_probability_no_exceedance_np(rup.probs_occur, poes, pnes)
-        return pnes
-    else:  # parametric
-        return tom.get_probability_no_exceedance(rup.occurrence_rate, poes)
+    def __init__(self, time_span, occurrence_rate):
+        self.time_span = time_span
+        self.occurrence_rate = occurrence_rate
 
 
-@compile("(float64[:], float64[:,:,:], float64, float64[:,:,:])")
-def calc_pnes(rates, poes, time_span, out):
+@compile(["(float64, float64[:], float64[:,:], float64)",
+          "(float64, float64[:], float64[:,:,:,:], float64)"])
+def get_pnes(rate, probs, poes, time_span):
     """
-    Compute probabilities of no exceedance by using the poisson distribution
-    (fast). Works by populating the "out" array.
+    :param rate: occurrence rate in case of a poissonian rupture
+    :param probs: probabilities of occurrence in the nonpoissonian case
+    :param poes: array of PoEs of shape 2D or 4D
+    :param time_span: time span in the poissonian case (0. for FatedTOM)
+
+    Fast way to return probabilities of no exceedence given an array
+    of PoEs and some parameter.
     """
-    for i, rate in enumerate(rates):
-        out[i] = -rate * time_span * poes[i]
-    numpy.exp(out, out)
+    if time_span == 0.:  # FatedTOM
+        return 1. - poes
+    elif len(probs) == 0:  # poissonian
+        return numpy.exp(-rate * time_span * poes)
+    else:
+        # Uses the formula
+        #
+        #    ∑ p(k|T) * p(X<x|rup)^k
+        #
+        # where `p(k|T)` is the probability that the rupture occurs k times
+        # in the time span `T`, `p(X<x|rup)` is the probability that a
+        # rupture occurrence does not cause a ground motion exceedance, and
+        # the summation `∑` is done over the number of occurrences `k`.
+        #
+        # `p(k|T)` is given by the attribute probs_occur and
+        # `p(X<x|rup)` is computed as ``1 - poes``.
+        pnes = numpy.full_like(poes, probs[0])
+        for p, prob in enumerate(probs[1:], 1):
+            pnes[:] += prob * (1 - poes) ** p
+        return pnes.clip(0., 1.)  # avoid numeric issues
 
 
-def get_probability_no_exceedance(ctx, poes, probs_or_tom):
+class NegativeBinomialTOM(BaseTOM):
     """
-    Vectorized version of :func:`get_probability_no_exceedance_rup`.
+    Negative Binomial temporal occurrence model.
+    """
+    def __init__(self, time_span, mu, alpha):
+        """
+        :param time_span:
+            The time interval of interest, in years.
+        :param occurrence_rate:
+            To initialize super_class (usually overriden by method
+            get_probability_no_exceedance())
+        :param parameters:
+            (list/np.ndarray) Parameters of the negbinom temporal model, in
+            form of mean rate/dispersion (μ / α)  Kagan and Jackson, (2010)
 
-    :param ctx: a recarray of length N
-    :param poes: an array of probabilities of length N
-    :param tom:
-        temporal occurrence model if the rupture is parametric,
-        list of N probability mass functions otherwise
-    """
-    pnes = numpy.zeros_like(poes)
-    if isinstance(probs_or_tom, FatedTOM):
-        for i, rate in enumerate(ctx.occurrence_rate):
-            pnes[i] = probs_or_tom.get_probability_no_exceedance(rate, poes[i])
-    elif isinstance(probs_or_tom, PoissonTOM):
-        calc_pnes(ctx.occurrence_rate, poes, probs_or_tom.time_span, pnes)
-    else:  # nonpoissonian
-        for i, probs_occur in enumerate(probs_or_tom):
-            set_probability_no_exceedance_np(probs_occur, poes[i], pnes[i])
-    return pnes
+                                    k
+                   Γ(τ + k)      μ           1
+            f(k) = -------- . -------- . --------
+                     Γ(τ)            k            1/α
+                              (μ + τ)    (1 + μ/τ)
 
+            where τ=1/α
+
+        """
+
+        super().__init__(time_span)
+        self.mu = mu
+        self.alpha = alpha
+        if numpy.any(self.mu <= 0) or numpy.any(self.alpha <= 0):
+            raise ValueError('Mean rate and rate dispersion must be greater '
+                             'than 0')
+        self.time_span = time_span
+
+    def get_probability_one_or_more_occurrences(self, mean_rate=None):
+        """
+
+        :param mean_rate:
+            The mean rate, or mean number of events per year
+        :return:
+            Float value between 0 and 1 inclusive.
+        """
+        if mean_rate is None:
+            mean_rate = self.mu
+        tau = 1 / self.alpha
+        theta = tau / (tau + (mean_rate * self.time_span))
+
+        return 1 - scipy.stats.nbinom.cdf(1, tau, theta)
+
+    def get_probability_n_occurrences(self, num):
+        """
+        Calculate the probability of occurrence  of ``num`` events in the
+        constructor's ``time_span``.
+
+        :param num:
+            Number of events
+        :return:
+            Probability of occurrence
+        """
+        tau = 1 / self.alpha
+        theta = tau / (tau + (self.mu * self.time_span))
+
+        return scipy.stats.nbinom.pmf(num, tau, theta)
+
+    def sample_number_of_occurrences(self, mean_rate=None, seed=None):
+        """
+        Draw a random sample from the distribution and return a number
+        of events to occur.
+
+        The method uses the numpy random generator, which needs a seed
+        in order to get reproducible results. If the seed is None, it
+        should be set outside of this method.
+
+        :param mean_rate:
+            The mean rate, or mean number of events per year
+        :param seed:
+            Random number generator seed
+        :return:
+            Sampled integer number of events to occur within model's
+            time span.
+        """
+        if mean_rate is None:
+            mean_rate = self.mu
+
+        tau = 1 / self.alpha
+        theta = tau / (tau + (mean_rate * self.time_span))
+
+        if isinstance(seed, int):
+            numpy.random.seed(seed)
+
+        return scipy.stats.nbinom.rvs(tau, theta)
+
+    def get_pmf(self, mean_rate, tol=1-1e-14, n_max=None):
+        """
+        :param mean_rate:
+            The average number of events per year.
+        :param tol:
+            Quantile value up to which calculate the pmf
+        :returns:
+            1D numpy array containing the probability mass distribution,
+            up to tolerance level.
+        """
+        # Gets dispersion from source object
+        alpha = self.alpha
+        # Recovers NB2 parametrization (tau/theta or n,p in literature)
+        tau = 1 / alpha
+        theta = tau / (tau + numpy.array(mean_rate).flatten()*self.time_span)
+        if not n_max:
+            n_max = numpy.max(
+                scipy.stats.nbinom.ppf(tol, tau, theta).astype(int))
+            if n_max < 4:
+                # minimum n_max for which the hazard equation is integrated,
+                # to avoid precision issues for probabilities of occur (<1e-6)
+                n_max = 4
+        pmf = scipy.stats.nbinom.pmf(
+            numpy.arange(0, n_max), tau, theta[:, None])
+        return pmf
+
+    def get_probability_no_exceedance(self, mean_rate, poes):
+        """
+        :param mean_rate:
+            The average number of events per year.
+        :param poes:
+            2D numpy array containing conditional probabilities that the
+            rupture occurrence causes a ground shaking value exceeding a
+            ground motion level at a site. First dimension represent sites,
+            second dimension intensity measure levels. ``poes`` can be obtained
+            calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`.
+        :returns:
+            2D numpy array containing probabilities of no exceedance. First
+            dimension represents sites, second dimension intensity measure
+            levels.
+
+        """
+
+        # Gets dispersion from source object
+        alpha = self.alpha
+        # Recovers NB2 parametrization (tau/theta or n,p in literature)
+        tau = 1 / alpha
+        theta = tau / (tau + mean_rate*self.time_span)
+
+        # Defines tol for the max quantile value, up to which the infinite series is calculated.
+        tol = 1 - 1e-14
+
+        n_max = scipy.stats.nbinom.ppf(tol, tau, theta)
+        pdf = scipy.stats.nbinom.pmf(numpy.arange(0, n_max), tau, theta)
+        poes_1 = 1 - poes
+        prob_no_exceed = numpy.zeros(poes.shape)
+        for k, prob in enumerate(pdf):
+            prob_no_exceed += prob * numpy.power(poes_1, k)
+
+        return prob_no_exceed
 
-@compile(["(float64[:], float64[:,:], float64[:,:])",
-          "(float64[:], float64[:,:,:,:], float64[:,:,:,:])"])
-def set_probability_no_exceedance_np(probs_occur, poes, pnes):
-    """
-    :param probs_occur: an array of probabilities
-    :param poes: an array of PoEs
-    :paran pnes: set an array of PNEs computed as ∑ p(k|T) * p(X<x|rup)^k
-    """
-    # Uses the formula
-    #
-    #    ∑ p(k|T) * p(X<x|rup)^k
-    #
-    # where `p(k|T)` is the probability that the rupture occurs k times
-    # in the time span `T`, `p(X<x|rup)` is the probability that a
-    # rupture occurrence does not cause a ground motion exceedance, and
-    # the summation `∑` is done over the number of occurrences `k`.
-    #
-    # `p(k|T)` is given by the attribute probs_occur and
-    # `p(X<x|rup)` is computed as ``1 - poes``.
-    arr = numpy.full_like(poes, probs_occur[0])
-    for p, v in enumerate(probs_occur[1:], 1):
-        arr += v * (1 - poes) ** p
-    pnes[:] = numpy.clip(arr, 0., 1.)  # avoid numeric issues
